unit ZLEKontrolaRealizacjiUnit;
{
 Copyright Unisoft Sp. z o.o.

Changes
---------------------------------------------------------------------------------------------
When       Who      What
---------------------------------------------------------------------------------------------
2004-03-29 gdeyk    Utworzone
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIDataModuleUnit, Db, UNIComponentsUnit, DBTables, DBClient;

type
  TZLEKontrolaRealizacjiDataModule = class(TUNIDataModule)
  protected
    FKontrolaWartosciowa: Boolean;
    FDataSet: TClientDataSet;
    FQuery: TUNIQuery;
    procedure AddFieldsToDataSet( oDataSet: TClientDataSet );
    procedure DefineColumns( oQuery: TUNIQuery); virtual; abstract;
    procedure AddRecordsToDataSet( oQueryFrom: TUNIQuery; oDataSetTo: TClientDataSet ); virtual; abstract;
    procedure CreateVirtualQuery(); virtual;
    procedure GetKeys; override;
    procedure InitializeGridQueryDefs; override;
    procedure OpenDataSets(); override;
  public
    constructor Create( AOwner: TComponent); override;
    destructor Destroy(); override;
    procedure Open; override;
    procedure ReOpenDataSets(); override;
    property KontrolaWartosciowa: Boolean read FKontrolaWartosciowa write FKontrolaWartosciowa;
    property Query: TUNIQuery read FQuery;
  end;

implementation

{$R *.DFM}

uses
  Dodatki,
  FormManagerUnit,
  KeyManagerUnit,
  QueryBuilderUnit,
  SQLManagerUnit,
  UNIFormUnit,
  UNIDBGridFormUnit,
  UNIStaleUnit,
  UNISysDefUnit,
  ASODefUnit,
  ASODataManagerExtUnit,
  ZLEDefUnit;

{ TZLEKontrolaRealizacjiDataModule }

procedure TZLEKontrolaRealizacjiDataModule.AddFieldsToDataSet(
  oDataSet: TClientDataSet);
var iIndex: Integer;
begin
  with oDataSet do begin
    for iIndex := 0 to GridColDefs.Count-1 do begin
      if FieldDefs.IndexOf(GridColDefs[iIndex].FieldName) = -1 then
        with FieldDefs.AddFieldDef do begin
          DataType := FieldTypeConvert(GridColDefs[iIndex].Typ);
          Size := 0;
          if DataType = ftString then Size := cZLEDlugoscPola_Nazwa;

          {Pole Uwagi musi byc dusze}
          if GridColDefs[iIndex].FieldName = sZLEILOPIS_UWAGI then
             Size := cZLEDlugoscPola_Uwagi;
          Name := GridColDefs[iIndex].FieldName;
          Required := false;
          DisplayName := GridColDefs[iIndex].FieldName;
        end;
    end;
    //wyrzucenie ju nie istniejcych pl
    for iIndex := FieldDefs.Count-1 downto 0 do begin
      if GridColDefs.IndexOfFieldName( FieldDefs.Items[iIndex].Name ) = -1 then
        FieldDefs.Delete(iIndex);
    end;
    IndexFieldNames := ''; 
  end;
end;

constructor TZLEKontrolaRealizacjiDataModule.Create(AOwner: TComponent);
begin
  inherited;
  FDataSet := nil;
  FQuery := nil;
end;

procedure TZLEKontrolaRealizacjiDataModule.CreateVirtualQuery;
var iIndex: Integer;
begin
  if FQuery <> nil then begin
    DefineColumns(FQuery);
    if FActiveQuery = nil then begin
      FDataSet := TClientDataSet.Create(nil);
      AddFieldsToDataSet( FDataSet );
      FDataSet.CreateDataSet();
      FActiveQuery := FDataSet;
    end else begin
      FDataSet.Close;
      AddFieldsToDataSet( FDataSet );
      FDataSet.CreateDataSet();
    end;
    if KontrolaWartosciowa then  //kontrola wartociowa
      for iIndex := 0 to FDataSet.Fields.Count-1 do begin
        if FDataSet.Fields[iIndex].DataType = ftFloat then
          (FDataSet.Fields[iIndex] as TNumericField).DisplayFormat := cMaskaKwotyExt;
      end
    else  //kontrola ilociowa
      for iIndex := 0 to FDataSet.Fields.Count-1 do begin
        if FDataSet.Fields[iIndex].DataType = ftFloat then
          (FDataSet.Fields[iIndex] as TNumericField).DisplayFormat := UNIGetMaskFromFormat( ASODM.vAsoFormatIlosci );
      end;
    AddRecordsToDataSet( FQuery, FDataSet );
  end;
end;

destructor TZLEKontrolaRealizacjiDataModule.Destroy;
begin
  inherited;
end;

procedure TZLEKontrolaRealizacjiDataModule.GetKeys;
begin
  with FKeys do begin
    AddPrimary(sUNIOKRES_ID);
  end;
end;

procedure TZLEKontrolaRealizacjiDataModule.InitializeGridQueryDefs;
begin
  CreateVirtualQuery();
  inherited;
end;

procedure TZLEKontrolaRealizacjiDataModule.Open;
begin
  //blokujemy otwarcie domylnego dataset'a
end;

procedure TZLEKontrolaRealizacjiDataModule.OpenDataSets;
var oSplit: TSplit;
    iIndex: Integer;
    sIndeksFields: String;
begin
  CreateVirtualQuery();
  if (Owner.InheritsFrom(TUNIDBGridForm)) then
    (Owner as TUNIDBGridForm).DBGrid.DataSource.DataSet := ActiveQuery;
  if FOrderByCols <> '' then begin
    oSplit := nil;
    sIndeksFields := '';
    try
      oSplit := TSplit.Create( FOrderByCols, ',', '' );
      for iIndex := 0 to oSplit.Count-1 do begin
        if (ActiveQuery as TClientDataSet).Fields.FindField(UtnijNazweKorelacji(oSplit[iIndex])) = nil then
          continue;
        if sIndeksFields <> '' then sIndeksFields := sIndeksFields+';';
        sIndeksFields := sIndeksFields + UtnijNazweKorelacji(oSplit[iIndex]);
      end;
    finally
      if oSplit <> nil then oSplit.Free;
    end;
    if Length(sIndeksFields) > 0 then
      (ActiveQuery as TClientDataSet).IndexFieldNames := sIndeksFields;
  end;
end;

procedure TZLEKontrolaRealizacjiDataModule.ReOpenDataSets;
begin
  inherited;
end;

end.
