unit ZLEOsobyUnit;
{
 Copyright Unisoft Sp. z o.o.

Modu dostarczajcy aktualizacje bazy.
Changes
---------------------------------------------------------------------------------------------
When       Who      What
---------------------------------------------------------------------------------------------
2007-11-19 Tomjen   Utworzone
---------------------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIDataModuleUnit, Db, UNIComponentsUnit, DBTables;

type
  TZLEOsobyDataModule = class(TUNIDataModule)
  protected
    procedure GetKeys; override;
    function  GetTableName: string; override;
    procedure InitializeGridQueryDefs; override;
  public
    procedure Post; override;
  private
    m_sZleOsobyId   : string;
    m_iZleOsobyIdx  : integer;
    m_sZleZasobyPracaId   : string;
    m_iZleZasobyPracaIdx  : integer;

    procedure ComposeConditions();
  end;

var
  ZLEOsobyDataModule: TZLEOsobyDataModule;

implementation

{$R *.DFM}

uses ZLEDefUnit, SQLManagerUnit, UNIStaleUnit, Dodatki,
     ZLEOsobyDBDetailFormUnit, UNISysDefUnit, SystemDefUnit,
     QueryBuilderUnit, KeyManagerUnit, UNIFormUnit;

procedure TZLEOsobyDataModule.GetKeys;
begin
  with FKeys do begin
    Clear;
    AddPrimary(sZLEOsoby_ID);

    if (Owner as TUNIForm).FormIsContained then
       Add(sZLEZasobyPraca_ID);
  end;
end;

function TZLEOsobyDataModule.GetTableName: string;
begin
  Result:=sZLEOsoby;
end;

procedure TZLEOsobyDataModule.InitializeGridQueryDefs;
begin
  with GridColDefs do begin
    Clear;
    AddRequiredAndUnvisible(sZLEOsoby_ID, 'K', '');
    AddRequiredAndUnvisible(sKDOsoba_ID, 'K', '');
    AddRequiredAndUnvisible(sZLEZasobyPraca_ID, 'K', '');
    Add(sKDOsoba_Imie,       'B','Imi',True,True, taLeftJustify,90);
    Add(sKDOsoba_Nazwisko,   'B','Nazwisko',True,True,  taLeftJustify,100);
  end;

  with GridTabDefs do begin
    Clear;
    Add('K',GetTableName);
    Add('B',sKDOSOBA);
  end;

  with GridJoinDefs do begin
    Clear;
    AddSimple( 'K', 'B', 'K.' + sKDOSOBA_ID +'=B.' + sKDOsoba_ID, qjkInner, True);
    m_iZleOsobyIdx := AddEmptyCondition();
    m_iZleZasobyPracaIdx := AddEmptyCondition();
  end;

  ComposeConditions();
  inherited;
end;

procedure TZLEOsobyDataModule.Post;
  var
    trans:boolean;
begin
  trans:= not SQLManager.InTransaction;
  if trans then SQLManager.StartTransaction;
  try
    inherited;
    if trans then SQLManager.Commit;
  except
    on EDMException do begin
      if trans then SQLManager.Rollback;
      if FLastDBError.SQLObjectName=sZLEOsoby+'1'+c_UNK then begin
        Komunikat( cError,'Istnieje ju taki element');
        {with owner as TZLEBrakiDBDetailForm do
           ActiveControl:= SymbolDBEdit;}
        abort;
      end else raise;
    end else begin
      if trans then SQLManager.Rollback;
      raise;
    end;
  end;

end;

procedure TZLEOsobyDataModule.ComposeConditions();
var
  oParams:        TKeyValues;
begin
  if (Owner as TUNIForm).FormIsContained then
     m_sZleZasobyPracaId := (Owner as TUNIForm).Parameters.Values[sZLEZasobyPraca_ID];

  if(not ParamIsNotEmpty(MasterKeyValues, sZLEOSOBY_ID, m_sZleOsobyId)) then begin
    oParams := TKeyValues.CreateEmpty();
    try
      (Owner as TUNIForm).GetParametersForForm((Owner as TUNIForm), oParams);
      ParamIsNotEmpty(oParams, sZLEOsoby_ID, m_sZleOsobyId);
    finally
      oParams.Free();
    end;
  end;  // if nie ma id w MasterKeyValues
  if(Length(m_sZleOsobyId) > 0) then begin
    GridJoinDefs.ExchangeCondition(m_iZleOsobyIdx, 'K.' + sZLEOsoby_ID  + ' = ' + m_sZleOsobyId, true);
  end else begin
    GridJoinDefs.ExchangeCondition(m_iZleOsobyIdx, '', false);
  end;
  
  if(Length(m_sZleZasobyPracaId) > 0) then begin
    GridJoinDefs.ExchangeCondition(m_iZleZasobyPracaIdx, 'K.' + sZLEZasobyPraca_ID  + ' = ' + m_sZleZasobyPracaId, true);
  end else begin
    GridJoinDefs.ExchangeCondition(m_iZleZasobyPracaIdx, '', false);
  end;
end;

end.
