CREATE OR REPLACE PACKAGE BODY zlexml_package AS

oResult Uniint_Package.TResult;
oResultNull Uniint_Package.TResult;

FUNCTION  ZapiszRaportZProdukcji(
  aRaportTypSymbol  IN VARCHAR2,
  xmlElement        IN Unixml_Package.XMLElement
) RETURN Uniint_Package.TResult IS

  datData     DATE;
  xmlPozycje  Unixml_Package.XMLElementList;
  xmlPozycja  Unixml_Package.XMLElement;
  sTemp       VARCHAR2(100);
  iZlecenieId INTEGER;
  iZasobId    INTEGER;
  iZmianaId   INTEGER;
  sPlanProdElId VARCHAR2(50);
  sRecepPozId VARCHAR2(50);
  dIlosc      DECIMAL;
  iLp         INTEGER;
  iBrakId     INTEGER;
  iPracaId    INTEGER;
  iRaportTId  integer;
  sPartnerId  VARCHAR2(21);
  sPartnerNr  VARCHAR2(50);
  dCzasPracy  Number(20, 6);

BEGIN
  oResult := oResultNull;
$IF UNIZSI_PACKAGE.ifLWP $THEN
  BEGIN
    select LWPRAPORTPRODTYP_ID
    into iRaportTId
    from LWPRAPORTPRODTYP
    where LWPRAPORTPRODTYP_SYMBOL = aRaportTypSymbol;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      iRaportTId  := null;
  END;
$ELSE
  iRaportTId  := null;
$END
  sTemp     := Unixml_Package.GetElementValue(xmlElement, MAGXML_PACKAGE.cMAG_DATA);
  sPartnerId:= Unixml_Package.GetElementValue(xmlElement, MAGXML_PACKAGE.cMAG_ID_OBCY);
  sPartnerNr:= Unixml_Package.GetElementValue(xmlElement, MAGXML_PACKAGE.cMAG_OBCY_NUMER);
  IF(Length(sTemp) > 10) THEN
      -- data podana cznie z godzin
    datData := To_Date(sTemp, 'YYYY-MM-DD HH24:MI:SS');
  ELSE
      -- sama data
    datData := sTemp;
  END IF;

  /*
    Na razie przyjmujemy, e kady rekord ma 8 godzin (480 min).
  */
  if(UNISYS_Package.GetSystemIniBoolean(0, 'ZLE', '', 'ZLEFormatGodzinLiczba', false)) then
    dCzasPracy  := 8;
  else
    dCzasPracy  := 480;
  end if;

    -- pozycje
  xmlPozycje  := Unixml_Package.SelectElements(xmlElement, MAGXML_PACKAGE.cMAG_POZYCJE || '/' || MAGXML_PACKAGE.cMAG_POZYCJA);

  FOR iIndex IN 0..Unixml_Package.GetElementCount(xmlPozycje) - 1 LOOP
    xmlPozycja  := Unixml_Package.GetElementByIndex(xmlPozycje, iIndex);
    iLp := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_LP), 1, 10);
    iZlecenieId := NULL;
    iZasobId    := NULL;
    iZmianaId   := NULL;
    sPlanProdElId := NULL;
    dIlosc      := NULL;
    iBrakId     := NULL;

      -- odczytanie zlecenia
    sTemp       := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_ZLECENIE_SYMBOL), 1, 100);
    IF(NOT sTemp IS NULL) THEN
      BEGIN
        SELECT Z.ZLEZLECENIE_ID INTO iZlecenieId FROM ZLEZLECENIE Z WHERE Upper(Z.ZLEZLECENIE_SYMBOL) = Upper(sTemp);
	    EXCEPTION
		    WHEN NO_DATA_FOUND THEN
          iZlecenieId := NULL;
      END;
    END IF;
    IF(iZlecenieId IS NULL) THEN
      oResult.Failure.Extend();
      oResult.Failure(oResult.Failure.Last).PartnerId     := sPartnerId;
      oResult.Failure(oResult.Failure.Last).PartnerNumber := sPartnerNr;
      IF(sTemp IS NULL) THEN
        oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie podano symbolu zlecenia';
      ELSE
        oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie odnaleziono zlecenia o symbolu: ' || sTemp;
      END IF;
      RETURN(oResult);
    END IF;
      -- odczytanie zasobu
    sTemp       := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_ZASOB_SYMBOL), 1, 100);
    IF(NOT sTemp IS NULL) THEN
      BEGIN
        SELECT Z.ZLEZASOB_ID INTO iZasobId FROM ZLEZASOB Z WHERE Upper(Z.ZLEZASOB_SYMBOL) = Upper(sTemp);
	    EXCEPTION
		    WHEN NO_DATA_FOUND THEN
          iZasobId := NULL;
      END;
    END IF;
    IF(iZasobId IS NULL) THEN
      oResult.Failure.Extend();
      oResult.Failure(oResult.Failure.Last).PartnerId     := sPartnerId;
      oResult.Failure(oResult.Failure.Last).PartnerNumber := sPartnerNr;
      IF(sTemp IS NULL) THEN
        oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie podano symbolu zasobu';
      ELSE
        oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie odnaleziono zasobu o symbolu: ' || sTemp;
      END IF;
      RETURN(oResult);
    END IF;
      -- odczytanie zmiany
    sTemp       := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_ZMIANA_SYMBOL), 1, 100);
    IF(NOT sTemp IS NULL) THEN
      BEGIN
  	    iZmianaId := ZLE_PACKAGE.DajIdZmianyDlaZasobu(iZasobId, sTemp, datData);
	    EXCEPTION
		    WHEN OTHERS THEN
          oResult.Failure.Extend();
          oResult.Failure(oResult.Failure.Last).PartnerId     := sPartnerId;
          oResult.Failure(oResult.Failure.Last).PartnerNumber := sPartnerNr;
          oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie odnaleziono zmiany o symbolu: ' || sTemp;
          RETURN(oResult);
      END;
    ELSE
      oResult.Failure.Extend();
      oResult.Failure(oResult.Failure.Last).PartnerId     := sPartnerId;
      oResult.Failure(oResult.Failure.Last).PartnerNumber := sPartnerNr;
      oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie podano symbolu zmiany';
      RETURN(oResult);
    END IF;
      -- odczytanie pozycji receptury lub elementu planu produkcji
    sRecepPozId   := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_RECEPTURAPOZ_ID), 1, 50);
    IF((sRecepPozId IS NOT NULL) AND (Length(sRecepPozId) > 2) AND (SubStr(sRecepPozId, 1, 2) = 'PL')) THEN
      sPlanProdElId := SubStr(sRecepPozId, 3);
      sRecepPozId   := NULL;
    END IF;
      -- odczytanie iloci
    sTemp       := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_ILOSC), 1, 30);
    IF(NOT sTemp IS NULL) THEN
      dIlosc  := To_Number(sTemp);
    ELSE
      oResult.Failure.Extend();
      oResult.Failure(oResult.Failure.Last).PartnerId     := sPartnerId;
      oResult.Failure(oResult.Failure.Last).PartnerNumber := sPartnerNr;
      oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' ie podano iloci';
      RETURN(oResult);
    END IF;
      -- odczytanie rodzaju braku
    sTemp   := SubStr(Unixml_Package.GetElementValue(xmlPozycja, MAGXML_PACKAGE.cMAG_BRAK_RODZAJ), 1, 100);
    IF(NOT sTemp IS NULL) THEN
      BEGIN
        SELECT Z.ZLEBRAKI_ID INTO iBrakId FROM ZLEBRAKI Z WHERE Upper(Z.ZLEBRAKI_SYMBOL) = Upper(sTemp);
	    EXCEPTION
		    WHEN NO_DATA_FOUND THEN
          iBrakId := NULL;
          oResult.Failure.Extend();
          oResult.Failure(oResult.Failure.Last).PartnerId     := sPartnerId;
          oResult.Failure(oResult.Failure.Last).PartnerNumber := sPartnerNr;
          oResult.Failure(oResult.Failure.Last).Reason := 'W pozycji ' || iLp || ' nie odnaleziono braku o symbolu: ' || sTemp;
          RETURN(oResult);
      END;
    END IF;

    IF(iBrakId IS NULL) THEN
      MERGE INTO ZLEZASOBYPRACA P
      USING (
        SELECT
          iZlecenieId AS ZLEZLECENIE_ID,
          iZasobId AS ZLEZASOB_ID,
          iZmianaId AS KDZMIANYDEF_ID,
          datData AS ZLEZASOBYPRACA_DATA,
          sPlanProdElId AS LWPPLANPRODEL_ID,
          sRecepPozId AS LWPRECEPPOZ_ID,
          dIlosc AS ZLEZASOBYPRACA_IL_PRACY,
          iRaportTId as LWPRAPORTPRODTYP_ID
        FROM
          dual
      ) N
      ON (
        P.ZLEZLECENIE_ID = N.ZLEZLECENIE_ID
        AND P.ZLEZASOB_ID = N.ZLEZASOB_ID
        AND P.KDZMIANYDEF_ID = N.KDZMIANYDEF_ID
        AND P.ZLEZASOBYPRACA_DATA = N.ZLEZASOBYPRACA_DATA
        AND Nvl(P.LWPPLANPRODEL_ID, 0) = Nvl(N.LWPPLANPRODEL_ID, 0)
        AND Nvl(P.LWPRECEPPOZ_ID, 0) = Nvl(N.LWPRECEPPOZ_ID, 0)
        AND P.ZLEZASOBYPRACA_STATUS = 'W'
        AND (
          (
            N.LWPRAPORTPRODTYP_ID IS NULL
            AND P.LWPRAPORTPRODTYP_ID IS NULL
          )
          OR P.LWPRAPORTPRODTYP_ID = N.LWPRAPORTPRODTYP_ID
        )
      )
      WHEN matched THEN
        UPDATE SET
          P.ZLEZASOBYPRACA_IL_PRACY = P.ZLEZASOBYPRACA_IL_PRACY + N.ZLEZASOBYPRACA_IL_PRACY,
          P.ZLEZASOBYPRACA_UZ_ZM_ID = UNISys_Package.UserId,
          P.ZLEZASOBYPRACA_DATA_ZM = SYSDATE
      WHEN NOT matched THEN
        INSERT (
          P.ZLEZASOBYPRACA_ID       , P.ZLEZLECENIE_ID        , P.ZLEZASOB_ID             , P.KDZMIANYDEF_ID,
          P.LWPPLANPRODEL_ID        , P.ZLEZASOBYPRACA_DATA   , P.ZLEZASOBYPRACA_IL_PRACY , P.ZLEZASOBYPRACA_CZAS_PRACY,
          P.LWPRECEPPOZ_ID          , P.ZLEZASOBYPRACA_STATUS , P.ZLEZASOBYPRACA_UZ_AU_ID , P.ZLEZASOBYPRACA_DATA_AU,
          P.LWPRAPORTPRODTYP_ID
        ) VALUES (
          SEQ_ZLEZASOBYPRACA.NEXTVAL, N.ZLEZLECENIE_ID        , N.ZLEZASOB_ID             , N.KDZMIANYDEF_ID,
          N.LWPPLANPRODEL_ID        , N.ZLEZASOBYPRACA_DATA   , N.ZLEZASOBYPRACA_IL_PRACY , dCzasPracy,
          N.LWPRECEPPOZ_ID          , 'W' , UNISys_Package.UserId, SysDate,
          N.LWPRAPORTPRODTYP_ID
        )
      ;
    ELSE
        -- odszukanie rekordu z prac
      BEGIN
        SELECT
          P.ZLEZASOBYPRACA_ID
        INTO
          iPracaId
        FROM
          ZLEZASOBYPRACA P
        WHERE
          P.ZLEZLECENIE_ID = iZlecenieId
          AND P.ZLEZASOB_ID = iZasobId
          AND P.KDZMIANYDEF_ID = iZmianaId
          AND P.ZLEZASOBYPRACA_DATA = datData
          AND Nvl(P.LWPPLANPRODEL_ID, 0) = Nvl(sPlanProdElId, 0)
          AND Nvl(P.LWPRECEPPOZ_ID, 0) = Nvl(sRecepPozId, 0)
          AND P.ZLEZASOBYPRACA_STATUS = 'W'
          and P.LWPRAPORTPRODTYP_ID = iRaportTId;
	    EXCEPTION
		    WHEN NO_DATA_FOUND THEN
          iPracaId := NULL;
      END;
      IF(iPracaId IS NULL) THEN
          -- trzeba doda nowy rekord z prac
        SELECT SEQ_ZLEZASOBYPRACA.NEXTVAL INTO iPracaId FROM dual;
        INSERT INTO ZLEZASOBYPRACA (
          ZLEZASOBYPRACA_ID       , ZLEZLECENIE_ID        , ZLEZASOB_ID             , KDZMIANYDEF_ID,
          LWPPLANPRODEL_ID        , ZLEZASOBYPRACA_DATA   , ZLEZASOBYPRACA_IL_PRACY , ZLEZASOBYPRACA_CZAS_PRACY,
          LWPRECEPPOZ_ID          , ZLEZASOBYPRACA_STATUS , ZLEZASOBYPRACA_UZ_AU_ID , ZLEZASOBYPRACA_DATA_AU,
          LWPRAPORTPRODTYP_ID
        ) VALUES (
          iPracaId                , iZlecenieId           , iZasobId                , iZmianaId,
          sPlanProdElId           , datData               , 0                       , dCzasPracy,
          sRecepPozId             , 'W'                   , UNISys_Package.UserId   , SysDate,
          iRaportTId
        );
      END IF;
        -- dodajemy rekord z brakami
      MERGE INTO ZLEZASOBYBRAKI B
      USING (
        SELECT
          iPracaId AS ZLEZASOBYPRACA_ID,
          iBrakId AS ZLEBRAKI_ID,
          dIlosc AS ZLEZASOBYBRAKI_IL
        FROM
          dual
      ) N
      ON (
        B.ZLEZASOBYPRACA_ID = N.ZLEZASOBYPRACA_ID
        AND B.ZLEBRAKI_ID = N.ZLEBRAKI_ID
      )
      WHEN matched THEN
        UPDATE SET
          B.ZLEZASOBYBRAKI_IL = B.ZLEZASOBYBRAKI_IL + N.ZLEZASOBYBRAKI_IL,
          B.ZLEZASOBYBRAKI_UZ_ZM_ID = UNISys_Package.UserId,
          B.ZLEZASOBYBRAKI_DATA_ZM = SYSDATE
      WHEN NOT matched THEN
        INSERT (
          B.ZLEZASOBYBRAKI_ID       , B.ZLEZASOBYBRAKI_IL       , B.ZLEBRAKI_ID           ,
          B.ZLEZASOBYPRACA_ID       , B.ZLEZASOBYBRAKI_UZ_AU_ID , B.ZLEZASOBYBRAKI_DATA_AU
        ) VALUES (
          SEQ_ZLEZASOBYBRAKI.NEXTVAL, N.ZLEZASOBYBRAKI_IL       , N.ZLEBRAKI_ID           ,
          N.ZLEZASOBYPRACA_ID       , UNISys_Package.UserId, SysDate
        )
      ;
    END IF;

  END LOOP;

  oResult.Success.Extend();
  oResult.Success(oResult.Success.Last).PartnerId     := sPartnerId;
  oResult.Success(oResult.Success.Last).PartnerNumber := sPartnerNr;
  RETURN oResult;
END;


END;