unit ZLEZasobZakresFormUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIRangeFormUnit, StdCtrls, Mask, UNIComponentsUnit, Menus, ToolWin,
  ComCtrls, Buttons, ExtCtrls;

type
  TZLEZasobyZakresForm = class(TUNIRangeForm)
    RodzajLabel: TUNILabel;
    ZasobRodzajComboBox: TUNIComboBox;
    JednostkaRangeLookupAliasEdit: TUNIRangeLookupAliasEdit;
    JednOrgLabel: TUNILabel;
    procedure ZasobRodzajComboBoxChange(Sender: TObject);
    procedure JednostkaRangeLookupAliasEditInitAliasInfo(Sender: TObject);
    function JednostkaRangeLookupAliasEditGetQuery: TUNIQuery;
  private
    bProcessData: Boolean;
    procedure InitializeControls();
    procedure InitializeObjectFields();
  public
    procedure ProcessRangeParamsToParameters; override;
    procedure PrepareParamsStep1; override;
    procedure PrepareParamsStep2; override;
    procedure ProcessData; override;
    procedure RestoreDefaults; override;
    procedure InternalOpen(); override;
  public
    destructor Destroy(); override;
    function GetHelpContextSuffix: String; override;
  end;

var
  ZLEZasobyZakresForm: TZLEZasobyZakresForm;

implementation

{$R *.DFM}

uses
  Dodatki,
  AliasManagerUnit,
  FormManagerUnit,
  SecurityManagerUnit,
  SQLManagerUnit,
  SystemDefUnit,
  UNISysDefUnit,
  UniSysDataManagerUnit,
  ASODefUnit,
  ASODataManagerExtUnit,
  BOKDefUnit,
  BOKDataManagerExtUnit,
  ZLEDefUnit,
  ZLEDataManagerUnit;

function TZLEZasobyZakresForm.GetHelpContextSuffix: String;
{ przyrostki do obs-ugi:
 [brak przyrostka]                        LWP                  2.2.5                Zasoby
 SZAS                                     zlecenia             2.7                  S-ownik zasobw
}
begin
  result := '';
end;

destructor TZLEZasobyZakresForm.Destroy;
begin
  inherited;
end;

procedure TZLEZasobyZakresForm.InitializeObjectFields;
begin
  bProcessData := False;
end;

procedure TZLEZasobyZakresForm.InitializeControls;
begin
  with ZasobRodzajComboBox do begin
    Items.Assign(cslZLERodzajZasobu);
    Items.Add('Wszystkie');
    DropDownCount := Items.Count;
  end;
end;

procedure TZLEZasobyZakresForm.PrepareParamsStep1;
begin
  inherited;
  with RangeParams do begin
    JednostkaRangeLookupAliasEdit.RangeParam := RangeParam(sUNIJEDNOSTKAORG_ID);
  end;
end;

procedure TZLEZasobyZakresForm.PrepareParamsStep2;
begin
  inherited;
end;

procedure TZLEZasobyZakresForm.ZasobRodzajComboBoxChange(
  Sender: TObject);
var
  iIndex: Integer;
begin
  iIndex := cslZLERodzajZasobu.IndexOf(ZasobRodzajComboBox.Text);
  if iIndex > -1 then
    RangeParam(sParam_ZLEZasobRodzID).Value := cslZLERodzajZasobu.ZwrocSkrot(iIndex)
  else
    RangeParam(sParam_ZLEZasobRodzID).Clear();
end;

procedure TZLEZasobyZakresForm.ProcessData;
var
  iIndex: Integer;
begin
  inherited;
  bProcessData := True;
  try
    iIndex := -1;
    if RangeParam(sParam_ZLEZasobRodzID).IsSet then
      iIndex := cslZLERodzajZasobu.IndexOfSkrot( RangeParam(sParam_ZLEZasobRodzID).Value );

    if iIndex = -1 then
      ZasobRodzajComboBox.ItemIndex := cslZLERodzajZasobu.Count
    else
      ZasobRodzajComboBox.ItemIndex := iIndex;

  finally
    bProcessData := False;
  end;
end;

procedure TZLEZasobyZakresForm.RestoreDefaults;
begin
  ClearRangeParams();
  PrepareParamsStep1;
  ProcessData();
end;

procedure TZLEZasobyZakresForm.InternalOpen;
begin
  InitializeObjectFields();
  InitializeControls();
  inherited;
end;


(****
function TZLEZasobyZakresForm.JednOrgDBLookupAliasEditGetDataField: String;
begin
  result := sUNIJEDNOSTKAORG_ID;
end;

function TZLEZasobyZakresForm.JednOrgDBLookupAliasEditGetQuery: TUNIQuery;
var
  sWarPoz: String;
begin

   sWarPoz := '';
   if not DMAQFbN(sUNIJEDNOSTKAORG_ID).IsNull then
      sWarPoz := sUNIJEDNOSTKAORG_ID + '=' + IntToStr(DMAQFbN(sUNIJEDNOSTKAORG_ID).AsInteger);


   Result := UNISysDataManager.UNIPokazIObsluzDane(sUNIJEDNOSTKAORG, '', false, false,
                     sUNISysAlias_JednostkaOrg_ID + ';' +
                     sUNISysAlias_JednostkaOrg_Symbol + ';' +
                     sUNISysAlias_JednostkaOrg_Nazwa,
                     vAktywnaFirma);
end;

procedure TZLEZasobyZakresForm.JednOrgDBLookupAliasEditInitAliasInfo(
  Sender: TObject);
begin
  with JednOrgDBLookupAliasEdit do begin
    AliasManager := UNISysDM.AliasManager;
    Alias_ID := sUNISysAlias_JednostkaOrg_ID;
    Alias_Name := sUNISysAlias_JednostkaOrg_Symbol;
    AdditionalAliases := sUNISysAlias_JednostkaOrg_Nazwa;
  end;
end;

function TZLEZasobyZakresForm.JednOrgDBLookupAliasEditSetData(
  AliasSrc: TDataSet): Boolean;
begin
  if AliasSrc <> nil then begin
    if (OpenMode in [omNew,omEdit])
       {and(JednOrgDBLookupAliasEdit.Initialized)} then begin
      DMAQFbN(sUNIJEDNOSTKAORG_ID).AsInteger := AliasSrc.FieldByName(sUNIJEDNOSTKAORG_ID).AsInteger;
    end;
  end else begin
     DMAQFbN(sUNIJEDNOSTKAORG_ID).Clear;
  end;
  result := True;
end;
*)

procedure TZLEZasobyZakresForm.ProcessRangeParamsToParameters;
begin
  inherited;
  with Parameters do begin
     Clear;
     Values[sParam_ZLEZasobRodzID] := RangeParam(sParam_ZLEZasobRodzID).Value;
     //Jednostka
     if RangeParam(sUNIJEDNOSTKAORG_ID).IsSet then
        Values[sUNIJEDNOSTKAORG_ID] := RangeParam(sUNIJEDNOSTKAORG_ID).CommaText
     else Values[sUNIJEDNOSTKAORG_ID] := '';

  end;
end;

procedure TZLEZasobyZakresForm.JednostkaRangeLookupAliasEditInitAliasInfo(
  Sender: TObject);
begin
  with JednostkaRangeLookupAliasEdit do begin
    AliasManager := UNISysDM.AliasManager;
    Alias_ID := sUNISysAlias_JednostkaOrg_ID;
    Alias_Name := sUNISysAlias_JednostkaOrg_Symbol;
   // AdditionalCondition := MultiFirmCond('', sMultiFirmCondEMPTY);
  end;
end;

function TZLEZasobyZakresForm.JednostkaRangeLookupAliasEditGetQuery: TUNIQuery;
var
  oSelectedId: TStringList;
begin
  oSelectedId := nil;
  if RangeParam(sUNIJEDNOSTKAORG_ID).Count > 1 then
     oSelectedId := RangeParam(sUNIJEDNOSTKAORG_ID);
  Result := UniSysDM.UNIPokazIObsluzDane(
    sUNIJEDNOSTKAORG,
    '',
    False,
    True,
    sUNISysAlias_JednostkaOrg_ID + ';' + sUNISysAlias_JednostkaOrg_Symbol,
    0,
    sUNIJEDNOSTKAORG_ID + '=' + iifs(RangeParam(sUNIJEDNOSTKAORG_ID).Value = '', '0', RangeParam(sUNIJEDNOSTKAORG_ID).Value),
    oSelectedId
  );
end;

initialization
  FormManager.RegisterForm( TZLEZasobyZakresForm, 'Zakres zasobw', sZLEModulID );

end.
