unit ZLEZlecenieMaterialDBGridFormUnit;
{
 Copyright Unisoft Sp. z o.o.

Changes
---------------------------------------------------------------------------------------------
When       Who      What
---------------------------------------------------------------------------------------------
2007-09-11 Adam     Utworzone
2008-08-26 Adam     Dodane wypenianie pla LWPRECEPPOZOPER_ID w materiaach podczas kopiowania z receptury
2008-09-10 Adam     Dodana obsuga innych jednostek miar ni podstawowa
2008-11-27 Adam     Poprawiona obsuga przeliczania jednostek miar (obsuga sposobu przeliczania)
2009-04-21 Adam     Dodana kontrola czy wypeniono konto materiaowe w materiaach zlece produkcyjnych
---------------------------------------------------------------------------------------------}
interface

uses
  UNIStaleUnit,  UNIDataManagerUnit,  UNIDataModuleUnit, UNIFormUnit, KeyManagerUnit,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIDBGridFormUnit, Db, DBTables, UNIComponentsUnit, URDataSet,
  UniDesigner, Menus, Grids, DBGrids, StdCtrls, ComCtrls, DBCtrls, ToolWin,
  ExtCtrls;

type
  TZLEZlecenieMaterialDBGridForm = class(TUNIDBGridForm)
    FunkcjeToolButton: TToolButton;
    FunkcjePopupMenu: TUNIPopupMenu;
    KopiujMatZRecepturyMenuItem: TMenuItem;
    ZamiennikMenuItem: TMenuItem;
    OrygMaterialMenuItem: TMenuItem;
    procedure KopiujMatZRecepturyMenuItemClick(Sender: TObject);
    procedure ZamiennikMenuItemClick(Sender: TObject);
    procedure OrygMaterialMenuItemClick(Sender: TObject);
    procedure DataSourceDataChange(Sender: TObject; Field: TField);
    procedure DBGridDrawColumnCell(Sender: TObject; const Rect: TRect;
      DataCol: Integer; Column: TColumn; State: TGridDrawState);
  public
    procedure GetParametersForForm(
                  a_oForm:              TUNIForm;
                  a_oParameters:        TKeyValues
              ); override;
      {
        Kopiuje materiay z przekazanej receptury.
        Iloci materiaw s przeliczane proporcjonalnie do wielkoci zlecenia i normy receptury
      }
    procedure KopiujMaterialyZReceptury(
                  const ac_iZlecenieId: integer;  // id zlecenia do ktrego maj by podpite materiay
                  const ac_iRecepturaId:integer;
                  const ac_dZlecIlosc:  double;
                  const ac_bProsOPotwierdzenie: boolean
              );
      {
        Pozwala wybra zamiennik dla aktualnego materiau.
        Ilo materiau jest przeliczana proporcjonalnie do wielkoci zlecenia i normy receptury
      }
    procedure WybierzZamiennik();
      {
        Przywraca dla aktualnej pozycji materia z receptury.
        Ilo materiau jest przeliczana proporcjonalnie do wielkoci zlecenia i normy receptury
      }
    procedure PrzywrocMaterialZReceptury();
      {
        Sprawdza czy dla materiau z aktualnego rekordu mona wybra zamiennik
      }
    function  MoznaWybracZamiennik(): boolean;
      {
        Sprawdza czy dla aktualnego rekordu mona przywrci materia z receptury
      }
    function  MoznaPrzywrocicMaterial():  boolean;
      {
        Wywietla detaila materiau i zwraca czy uytkownik zatwierdzi go czy te anulowa.
        Zwraca TRUE jeeli uytkownik zatwierdzi formatk przez Zatwierd.
      }
    function  EdytujMaterial(): boolean;

  protected
    function  CreateDataModule(
                  const AOpenMode:  TOpenMode
              ): TUNIDataModule; override;
    procedure View; override;
    procedure New; override;
    procedure Open; override;
    procedure Delete; override;

  end;

implementation
uses
  UniqueKeyUnit,
  FormManagerUnit,
  SQLManagerUnit,
  Dodatki,
  ZLEZlecenieMaterialUnit,
  ZLEZlecenieMaterialDBDetailFormUnit,
  ZLEZlecenieDBDetailFormUnit,
  ZLEDataManagerUnit,
  LWPDataManagerExtUnit,
  MAGDefUnit,
  ZAODefUnit,
  UNISysDefUnit,
  ASODefUnit,
  LWPDefUnit,
  ZLEDefUnit
  ;

{$R *.DFM}

const
  csError_AsortBrakJednostki      = 'Asortyment %s nie ma jednostki %s';
  csError_AsortBrakWspolczynnika  = 'Asortyment %s nie ma podanego wspczynnik a dla jednostki %s';

procedure TZLEZlecenieMaterialDBGridForm.GetParametersForForm(
  a_oForm:          TUNIForm;
  a_oParameters:    TKeyValues
);
begin
  if(Owner is TUNIForm) then begin
    (Owner as TUNIForm).GetParametersForForm(self, a_oParameters);
  end else if(Parameters <> nil) then
    a_oParameters.Assign(Parameters);
end;

procedure TZLEZlecenieMaterialDBGridForm.KopiujMaterialyZReceptury(
  const ac_iZlecenieId: integer;  // id zlecenia do ktrego maj by podpite materiay
  const ac_iRecepturaId:integer;
  const ac_dZlecIlosc:  double;
  const ac_bProsOPotwierdzenie: boolean
);
  procedure DopiszMaterial(
    const ac_oQuery:  TUNIQuery
  );
  var
    i:      integer;
    dIlosc: double;
  begin
      // sprawdzenie ktra jednostka jest wybrana w recepturze i odpowiednie przeliczenie iloci
    if(ac_oQuery.Fields[18].AsInteger = ac_oQuery.Fields[20].AsInteger) then begin
        // wybrana podstawowa jednostka
      dIlosc  := ac_oQuery.FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat;
    end else if(ac_oQuery.Fields[18].AsInteger = ac_oQuery.Fields[21].AsInteger) then begin
        // wybrana druga jednostka
      if(ac_oQuery.Fields[22].AsInteger = 0) then
        raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [ac_oQuery.Fields[1].AsString, '2']));
      if(ac_oQuery.Fields[25].AsString = sASOCONST_KIER_PRZEL1_ID) then
        dIlosc  := (ac_oQuery.FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat * ac_oQuery.Fields[22].AsFloat)
      else
        dIlosc  := (ac_oQuery.FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat / ac_oQuery.Fields[22].AsFloat);
    end else if(ac_oQuery.Fields[18].AsInteger = ac_oQuery.Fields[23].AsInteger) then begin
        // wybrana trzecia jednostka
      if(ac_oQuery.Fields[24].AsInteger = 0) then
        raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [ac_oQuery.Fields[1].AsString, '3']));
      if(ac_oQuery.Fields[26].AsString = sASOCONST_KIER_PRZEL1_ID) then
        dIlosc  := (ac_oQuery.FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat * ac_oQuery.Fields[24].AsFloat)
      else
        dIlosc  := (ac_oQuery.FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat / ac_oQuery.Fields[24].AsFloat);
    end else begin
      raise EOF_ZLEException.Create(Format(csError_AsortBrakJednostki, [ac_oQuery.Fields[1].AsString, ac_oQuery.Fields[19].AsString]));
    end;
    with(DataModule.ActiveQuery as TVirtualDataSet) do begin
      VState  := vsInternalInsert;
      try
        Insert();
        FieldByName(sZLEZLECENIEMAT_ID).AsInteger     := UniqueKey[sZLEZLECENIEMAT];
        FieldByName(sZLEZLECENIE_ID).AsInteger        := ac_oQuery.FieldByName(sZLEZLECENIE_ID).AsInteger;
        FieldByName(sLWPRECEPPOZMATERIAL_ID).AsInteger:= ac_oQuery.FieldByName(sLWPRECEPPOZMATERIAL_ID).AsInteger;
        FieldByName(sASOASORT_ID).AsInteger           := ac_oQuery.Fields[8].AsInteger;
        FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat    := dIlosc;
        if(not ac_oQuery.FieldByName(sASOASORTKMAT_ID).IsNull) then
          FieldByName(sASOASORTKMAT_ID).AsInteger       := ac_oQuery.FieldByName(sASOASORTKMAT_ID).AsInteger;
        if(ac_oQuery.FieldByName(sLWPRECEPPOZOPER_ID).AsInteger <> 0) then
          FieldByName(sLWPRECEPPOZOPER_ID).AsInteger  := ac_oQuery.FieldByName(sLWPRECEPPOZOPER_ID).AsInteger;
        SetTechnicalFieldsOnInsert();
        UpdateModifiedRecordFlagsForAddedRecord();
        for i := 1 to ac_oQuery.Fields.Count-1 do begin
          if((ac_oQuery.Fields[i].FullName <> sZLEZLECENIEMAT_ILOSC) and (FindField(ac_oQuery.Fields[i].FullName) <> nil)) then
            FieldByName(ac_oQuery.Fields[i].FullName).AsString := ac_oQuery.Fields[i].AsString;
        end;  // for i := 1 to ac_oQuery.Fields.Count-1
        Post();
      finally
        VState := vsNormal;
      end;
    end;  // with(DataModule.ActiveQuery as TVirtualDataSet)
  end;  // procedure DopiszMaterial

var
  oQuery:   TUNIQuery;
  bUsun:    boolean;
begin
  if(
    (not ac_bProsOPotwierdzenie) or
    (Komunikat(cConfirmation, 'Przywrci materiay z receptury?', MB_YESNO + MB_ICONQUESTION) = mrYes)
  ) then begin
    bUsun := false;
    if(not DataModule.ActiveQuery.IsEmpty()) then begin
      bUsun := (Komunikat(
        cConfirmation,
        'Usuna istniejce materiay ze zlecenia?',
        MB_YESNO + MB_DEFBUTTON1 + MB_ICONQUESTION
      ) = mrYes);
    end;
    if(bUsun) then begin
      DataModule.ActiveQuery.First();
      while(not DataModule.ActiveQuery.Eof) do begin
        DataModule.ActiveQuery.Delete();
      end;
    end;  // if(bUsun)

    oQuery  := SQLManager.CreateQuery(
      SQLManager.GetSelectWithLeftOuterJoins(
{0}     '(' +
          '(' + UNIFloatToStr(ac_dZlecIlosc) + ' * RECPM.' + sLWPRECEPPOZMATERIAL_ILOSC + ') / REC.' + sLWPRECEPTURA_ILOSC +
        ') AS ' + sZLEZLECENIEMAT_ILOSC  + ', ' +
{1}     'A.'  + sASOASORT_SYMBOL  + ', A.'  + sASOASORT_NAZWA + ', ' +
{3}     'A.'  + sASOASORT_INDEKS  + ', A.'  + sASOASORT_PKWIU + ', ' +
{5}     'G.'  + sASOGRAS_SYMBOL   + ', G.'  + sASOGRAS_NAZWA  + ', ' +
{7}     'JM.' + sUNIJM_SYMBOL     + ', A.'  + sASOASORT_ID    + ', ' +
{9}     'RECP.'+sLWPRECEPPOZ_ID   + ', RECPM.'+ sLWPRECEPPOZMATERIAL_ID + ', ' +
{11}    IntToStr(ac_iZlecenieId)  + ' as ' + sZLEZLECENIE_ID + ', ' +
{12}    'AKM.'+ sASOASORTKMAT_ID  + ', KM.' + sASOKMAT_SYMBOL + ', ' +
{14}    'KM.' + sASOKMAT_NAZWA    + ', 0 AS ' + sLWPRECEPPOZOPER_ID + ', ' +
{16}    'NULL as ' + sLWPRECEPPOZOPER_NAZWA  + ', NULL as ' + sLWPRECEPPOZOPER_OPIS + ', ' +
{18}    'RECPM.' + sUNIJM_ID      + ', JMRP.'+ sUNIJM_SYMBOL      + ', ' +
{20}    'A.'  + sUNIJM_ID1        + ', ' +
{21}    'A.'  + sUNIJM_ID2        + ', A.'  + sASOASORT_WSPJM1JM2 + ', ' +
{23}    'A.'  + sUNIJM_ID3        + ', A.'  + sASOASORT_WSPJM1JM3 + ', ' +
{25}    'A.'  + sASOASORT_ZJM1NAJM2+', A.'  + sASOASORT_ZJM1NAJM3,
        [
          sLWPRECEPTURA , sLWPRECEPPOZ  ,
          sLWPRECEPPOZMATERIAL, sASOASORT   , sASOGRAS,
          sUNIJM  , sASOASORTKMAT , sASOKMAT, sUNIJM
        ],
        [
          'REC' , 'RECP',
          'RECPM' , 'A' , 'G'   ,
          'JM'  , 'AKM' , 'KM', 'JMRP'
        ],
        [
          'A.'    + sASOGRAS_ID       + ' = G.'   + sASOGRAS_ID ,
          'A.'    + sUNIJM_ID1        + ' = JM.'  + sUNIJM_ID   ,
          'RECPM.'+ sASOASORTKMAT_ID  + ' = AKM.' + sASOASORTKMAT_ID,
          'AKM.'  + sASOKMAT_ID       + ' = KM.'  + sASOKMAT_ID,
          'RECPM.'+ sUNIJM_ID         + ' = JMRP.'+ sUNIJM_ID
        ],
        'REC.'  + sLWPRECEPTURA_ID  + ' = RECP.'+ sLWPRECEPTURA_ID+ ' and ' +
        'RECP.' + sLWPRECEPPOZ_ID   + ' = RECPM.'+sLWPRECEPPOZ_ID + ' and ' +
        'RECPM.'+ sASOASORT_ID      + ' = A.'   + sASOASORT_ID    + ' and ' +
        'REC.'+ sLWPRECEPTURA_ID+ ' = ' + IntToStr(ac_iRecepturaId)
      ) + ' union all ' +
      SQLManager.GetSelectWithLeftOuterJoins(
{0}     '(' +
          '(' + UNIFloatToStr(ac_dZlecIlosc) + ' * OPMAT.' + sLWPRECEPPOZOPERMAT_ILOSC + ') / REC.' + sLWPRECEPTURA_ILOSC +
        ') AS ' + sZLEZLECENIEMAT_ILOSC  + ', ' +
{1}     'A.'  + sASOASORT_SYMBOL  + ', A.'  + sASOASORT_NAZWA + ', ' +
{3}     'A.'  + sASOASORT_INDEKS  + ', A.'  + sASOASORT_PKWIU + ', ' +
{5}     'G.'  + sASOGRAS_SYMBOL   + ', G.'  + sASOGRAS_NAZWA  + ', ' +
{7}     'JM.' + sUNIJM_SYMBOL     + ', A.'  + sASOASORT_ID    + ', ' +
{9}     'RECP.'+sLWPRECEPPOZ_ID   + ', NULL AS '+ sLWPRECEPPOZMATERIAL_ID + ', ' +
{11}    IntToStr(ac_iZlecenieId)  + ' as ' + sZLEZLECENIE_ID + ', ' +
{12}    'AKM.'+ sASOASORTKMAT_ID  + ', KM.' + sASOKMAT_SYMBOL + ', ' +
{14}    'KM.' + sASOKMAT_NAZWA    + ', OP.' + sLWPRECEPPOZOPER_ID + ', '+
{16}    'OP.' + sLWPRECEPPOZOPER_NAZWA  + ', OP.' + sLWPRECEPPOZOPER_OPIS + ', ' +
{18}    'OPMAT.' + sUNIJM_ID      + ', JMRP.'+ sUNIJM_SYMBOL      + ', '    +
{20}    'A.'  + sUNIJM_ID1        + ', ' +
{21}    'A.'  + sUNIJM_ID2        + ', A.'  + sASOASORT_WSPJM1JM2 + ', ' +
{23}    'A.'  + sUNIJM_ID3        + ', A.'  + sASOASORT_WSPJM1JM3 + ', ' +
{25}    'A.'  + sASOASORT_ZJM1NAJM2+', A.'  + sASOASORT_ZJM1NAJM3,
        [
          sLWPRECEPTURA , sLWPRECEPPOZ  ,
          sLWPRECEPPOZOPER, sLWPRECEPPOZOPERMAT, sASOASORT, sASOGRAS,
          sUNIJM  , sASOASORTKMAT , sASOKMAT  , sUNIJM
        ],
        [
          'REC' , 'RECP',
          'OP' , 'OPMAT', 'A' , 'G'   ,
          'JM'  , 'AKM' , 'KM', 'JMRP'
        ],
        [
          'A.'    + sASOGRAS_ID       + ' = G.'   + sASOGRAS_ID ,
          'A.'    + sUNIJM_ID1        + ' = JM.'  + sUNIJM_ID   ,
          'OPMAT.'+ sASOASORTKMAT_ID  + ' = AKM.' + sASOASORTKMAT_ID,
          'AKM.'  + sASOKMAT_ID       + ' = KM.'  + sASOKMAT_ID,
          'OPMAT.'+ sUNIJM_ID         + ' = JMRP.'+ sUNIJM_ID
        ],
        'REC.'  + sLWPRECEPTURA_ID  + ' = RECP.'+ sLWPRECEPTURA_ID+ ' and ' +
        'RECP.' + sLWPRECEPPOZ_ID   + ' = OP.'  + sLWPRECEPPOZ_ID + ' and ' +
        'OP.'   + sLWPRECEPPOZOPER_ID + ' = OPMAT.' + sLWPRECEPPOZOPER_ID + ' and ' +
        'OPMAT.'+ sASOASORT_ID      + ' = A.'   + sASOASORT_ID    + ' and ' +
        'REC.'+ sLWPRECEPTURA_ID+ ' = ' + IntToStr(ac_iRecepturaId)
      )
    );
    try
      oQuery.Open();
      while(not oQuery.Eof) do begin
        DopiszMaterial(oQuery);
        oQuery.Next();
      end;  // while(not oQuery.Eof)
    finally
      oQuery.Free();
    end;
  end;  // if zgodna na przywrcenie materiaw
end;

procedure TZLEZlecenieMaterialDBGridForm.WybierzZamiennik();

  procedure WybierzZReceptury();
  var
    oQuery:   TUNIQuery;
    oQueryKM: TUNIQuery;
    oDetail:  TZLEZlecenieMaterialDBDetailForm;
    dIlosc:   double;
    sId:      string;
  begin
      // UWAGA: w tym zapytaniu symbol jednostki to jednostka materiau (nie musi to by podstawowa jednostka asortymentu)
    oQuery  := LWPDM.LWPPokazIObsluzDane(sLWPRECEPPOZMATERIALZAM,
      sLWPAlias_RecepturaPozMaterialZam_Id,
      '@' + sLWPAlias_RecepturaPozMaterial_Id + ' = ' + DMAQFbN(sLWPRECEPPOZMATERIAL_ID).AsString,
      true, true, false
    );
    try
      if(oQuery <> nil) then begin
        sId := oQuery.Fields[0].AsString;
        FreeAndNil(oQuery);
        oQuery  := SQLManager.CreateQuery(
          'select ' +
{0}         'A.'    + sASOASORT_ID        + ', RPMZ.' + sLWPRECEPPOZMATERIALZAM_ILOSC + ', ' +
{2}         'R.'    + sLWPRECEPTURA_ILOSC + ', A.'    + sASOASORT_SYMBOL  + ', ' +
{4}         'A.'    + sASOASORT_NAZWA     + ', A.'    + sASOASORT_PKWIU   + ', ' +
{6}         'A.'    + sASOASORT_INDEKS    + ', AG.'   + sASOGRAS_SYMBOL   + ', ' +
{8}         'AG.'   + sASOGRAS_NAZWA      + ', JM.'   + sUNIJM_SYMBOL     + ', ' +
{10}        'JM.'   + sUNIJM_NAZWA        + ', RP.'   + sLWPRECEPPOZ_ID   + ', ' +
{12}        'RPMZ.' + sLWPRECEPPOZMATERIAL_ID + ', RPMZ.' + sLWPRECEPPOZMATERIALZAM_ID  + ', ' +
{14}        'RPMZ.' + sUNIJM_ID           + ', A.'    + sUNIJM_ID1        + ', ' +
{16}        'A.'    + sUNIJM_ID2          + ', A.'    + sASOASORT_WSPJM1JM2 + ', ' +
{18}        'A.'    + sUNIJM_ID3          + ', A.'    + sASOASORT_WSPJM1JM3 + ', ' +
{20}        'JMZ.'  + sUNIJM_SYMBOL       + ', ' +
{21}        'A.'    + sASOASORT_ZJM1NAJM2 + ', A.'    + sASOASORT_ZJM1NAJM3 +
          ' from '  +
            ' %s RPMZ'  +
              ' inner join %s A'  +
                ' on RPMZ.' + sASOASORT_ID  + ' = A.' + sASOASORT_ID  +
              ' left outer join %s JM'  +
                ' on A.'  + sUNIJM_ID1  + ' = JM.'  + sUNIJM_ID +
              ' left outer join %s AG'  +
                ' on A.'  + sASOGRAS_ID + ' = AG.'  + sASOGRAS_ID +
              ' inner join %s RPM' +
                ' on RPMZ.' + sLWPRECEPPOZMATERIAL_ID + ' = RPM.' + sLWPRECEPPOZMATERIAL_ID +
              ' inner join %s RP' +
                ' on RPM.'  + sLWPRECEPPOZ_ID + ' = RP.'  + sLWPRECEPPOZ_ID +
              ' inner join %s R'  +
                ' on RP.'   + sLWPRECEPTURA_ID+ ' = R.'   + sLWPRECEPTURA_ID+
              ' left outer join %s JMZ' +
                ' on RPMZ.' + sUNIJM_ID + ' = JMZ.' + sUNIJM_ID +
          ' where ' +
            'RPMZ.' + sLWPRECEPPOZMATERIALZAM_ID  + ' = ' + sId,
          [
            sLWPRECEPPOZMATERIALZAM , sASOASORT, sUNIJM, sASOGRAS,
            sLWPRECEPPOZMATERIAL    , sLWPRECEPPOZ, sLWPRECEPTURA,
            sUNIJM
          ]
        );
        oQuery.Open();
        dIlosc  := (
            ((Owner as TZLEZlecenieDBDetailForm).DMAQFbN(sZLEZLECENIE_ILOSC).AsFloat * oQuery.Fields[1].AsFloat) /
              oQuery.Fields[2].AsFloat
        );
        if(oQuery.Fields[14].AsInteger = oQuery.Fields[15].AsInteger) then begin
            // podstawowa jednostka
          dIlosc  := dIlosc;
        end else if(oQuery.Fields[14].AsInteger = oQuery.Fields[16].AsInteger) then begin
            // druga jednostka
          if(oQuery.Fields[17].AsInteger = 0) then
            raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[3].AsString, '2']));
          if(oQuery.Fields[21].AsString = sASOCONST_KIER_PRZEL1_ID) then
            dIlosc  := dIlosc * oQuery.Fields[17].AsFloat
          else
            dIlosc  := dIlosc / oQuery.Fields[17].AsFloat;
        end else if(oQuery.Fields[14].AsInteger = oQuery.Fields[18].AsInteger) then begin
            // trzecia jednostka
          if(oQuery.Fields[19].AsInteger = 0) then
            raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[3].AsString, '3']));
          if(oQuery.Fields[22].AsString = sASOCONST_KIER_PRZEL1_ID) then
            dIlosc  := dIlosc * oQuery.Fields[19].AsFloat
          else
            dIlosc  := dIlosc / oQuery.Fields[19].AsFloat;
        end else begin
          raise EOF_ZLEException.Create(Format(csError_AsortBrakJednostki, [oQuery.Fields[3].AsString, oQuery.Fields[20].AsString]));
        end;
        oDetail := (FormManager.OpenDBDetail(
          self,
          'TZLEZlecenieMaterialDBDetailForm',
          omEdit + omHidden + omOverVirtual
        ) as TZLEZlecenieMaterialDBDetailForm);
        try
          oDetail.DMAQFbN(sASOASORT_ID).AsInteger       := oQuery.Fields[0].AsInteger;
          oDetail.DMAQFbN(sZLEZLECENIEMAT_ILOSC).AsFloat:= dIlosc;
          oDetail.ShowAsort(oQuery);
          oQueryKM  := SQLManager.CreateQuery(
            'select ' +
              'KM.' + sASOKMAT_ID + ', KM.' + sASOKMAT_SYMBOL + ', ' +
              'KM.' + sASOKMAT_NAZWA  + ', RPMZ.' + sASOASORTKMAT_ID  +
            ' from ' +
              '%s RPMZ' +
                ' inner join %s AKM'  +
                  ' on RPMZ.' + sASOASORTKMAT_ID  + ' = AKM.' + sASOASORTKMAT_ID  +
                ' inner join %s KM' +
                  ' on AKM.'  + sASOKMAT_ID + ' = KM.'  + sASOKMAT_ID +
            ' where ' +
              'RPMZ.' + sLWPRECEPPOZMATERIALZAM_ID  + ' = ' + oQuery.Fields[13].AsString,
            [sLWPRECEPPOZMATERIALZAM, sASOASORTKMAT, sASOKMAT]
          );
          oQueryKM.Open();
          if(not oQueryKM.IsEmpty()) then begin
            if(not oQueryKM.Fields[3].IsNull) then
              oDetail.DMAQFbN(sASOASORTKMAT_ID).AsInteger := oQueryKM.Fields[3].AsInteger
            else
              oDetail.DMAQFbN(sASOASORTKMAT_ID).Clear();
            oDetail.ShowKontoMaterialowe(oQueryKM);
            oQueryKM.Free();
          end;  // if(not oQueryKM.IsEmpty())
          oDetail.ShowModal();
        finally
          oDetail.Free();
        end;
      end;  // if(oQuery <> nil)
    finally
      FreeAndNil(oQuery);
    end;
  end;  // procedure WybierzZReceptury

  procedure WybierzZOperacji();
  var
    iOperId:    integer;
    iZlecId:    integer;
    i:          integer;
    oQuery:     TUNIQuery;
    oQueryJM:   TUNIQuery;
    dIlosc:     double;
    sJMSymbol:  string;
    sJMNazwa:   string;
  begin
    oQuery  := nil;
    oQueryJM:= nil;
    DataModule.ActiveQuery.DisableControls();
    try
      oQueryJM:= SQLManager.CreateQuery(
        'select JM.'  + sUNIJM_SYMBOL + ', JM.'  + sUNIJM_NAZWA + ' from %s JM where JM.' + sUNIJM_ID + ' = :' + sUNIJM_ID,
        [sUNIJM]
      );
      oQuery  := LWPDM.LWPPokazIObsluzDane(sLWPRECEPPOZOPERZAM,
  {0}   sLWPAlias_ASOAsortOperZam_Id+ ';' + sLWPAlias_RecepturaPozOperZamMat_Ilosc + ';' +
  {2}   sLWPAlias_Receptura_Ilosc   + ';' + sLWPAlias_ASOAsortOperZam_Symbol+ ';' +
  {4}   sLWPAlias_ASOAsortOperZam_Nazwa + ';' + sLWPAlias_ASOAsortOperZam_PKWIU + ';' +
  {6}   sLWPAlias_ASOAsortOperZam_Indeks+ ';' + sLWPAlias_ASOGrasOperZam_Symbol + ';' +
  {8}   sLWPAlias_ASOGrasOperZam_Nazwa  + ';' + sLWPAlias_UNIJMOperZam_Symbol   + ';' +
  {10}  sLWPAlias_UNIJMOperZam_Nazwa    + ';' + sLWPAlias_RecepturaPoz_Id   + ';' +
  {12}  sLWPAlias_RecepturaPozOperZamMat_Id  + ';' +
  {13}  sLWPAlias_ASOAsortKMatOperZam_Id+ ';' + sLWPAlias_ASOKMatOperZam_Symbol + ';' +
  {15}  sLWPAlias_ASOKMatOperZam_Nazwa  + ';' + sLWPAlias_RecepturaPozOperZam_Id+ ';' +
  {17}  sLWPAlias_RecepturaPozOper_Id   + ';' +
  {18}  sLWPAlias_RecepturaPozOperZamMat_JM_Id + ';' + sLWPAlias_ASOAsortOperZam_JM1_Id+ ';' +
  {20}  sLWPAlias_ASOAsortOperZam_JM2_Id+ ';' + sLWPAlias_ASOAsortOperZam_JM2_Wsp + ';' +
  {22}  sLWPAlias_ASOAsortOperZam_JM3_Id+ ';' + sLWPAlias_ASOAsortOperZam_JM3_Wsp,
        '@' + sLWPAlias_RecepturaPozOper_Id + ' = ' + DMAQFbN(sLWPRECEPPOZOPER_ID).AsString,
        true, true, false
      );
      if(oQuery <> nil) then begin
          // najpierw usuwamy aktualne rekordy z biecej operacji
        iOperId := DMAQFbN(sLWPRECEPPOZOPER_ID).AsInteger;
        iZlecId := DMAQFbN(sZLEZLECENIE_ID).AsInteger;
        DataModule.ActiveQuery.First();
        while(not DataModule.ActiveQuery.Eof) do begin
          if(DMAQFbN(sLWPRECEPPOZOPER_ID).AsInteger = iOperId) then begin
            DataModule.ActiveQuery.Delete();
          end else
            DataModule.ActiveQuery.Next();
        end;  // while(not DataModule.ActiveQuery.Eof)
          // dopisujemy materiay operacji
        while(not oQuery.Eof) do begin
          dIlosc  := (
            ((self.Owner as TZLEZlecenieDBDetailForm).DMAQFbN(sZLEZLECENIE_ILOSC).AsFloat * oQuery.Fields[1].AsFloat) /
              oQuery.Fields[2].AsFloat
          );
          oQueryJM.ParamByName(sUNIJM_ID).AsInteger := oQuery.Fields[19].AsInteger;
          if(oQuery.Fields[18].AsInteger = oQuery.Fields[19].AsInteger) then begin
              // podstawowa jednostka
            dIlosc    := dIlosc;
            sJMSymbol := oQuery.Fields[9].AsString;
            sJMNazwa  := oQuery.Fields[10].AsString;
          end else if(oQuery.Fields[18].AsInteger = oQuery.Fields[20].AsInteger) then begin
              // druga jednostka
            if(oQuery.Fields[21].AsInteger = 0) then
              raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[3].AsString, '2']));
            dIlosc  := dIlosc / oQuery.Fields[21].AsFloat;
            oQueryJM.Open();
            sJMSymbol := oQueryJM.Fields[0].AsString;
            sJMNazwa  := oQueryJM.Fields[1].AsString;
            oQueryJM.Close();
          end else if(oQuery.Fields[18].AsInteger = oQuery.Fields[22].AsInteger) then begin
              // trzecia jednostka
            if(oQuery.Fields[23].AsInteger = 0) then
              raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[3].AsString, '3']));
            dIlosc  := dIlosc / oQuery.Fields[23].AsFloat;
            oQueryJM.Open();
            sJMSymbol := oQueryJM.Fields[0].AsString;
            sJMNazwa  := oQueryJM.Fields[1].AsString;
            oQueryJM.Close();
          end else begin
            raise EOF_ZLEException.Create(Format(csError_AsortBrakJednostki, [oQuery.Fields[3].AsString, oQuery.Fields[9].AsString]));
          end;
          with(DataModule.ActiveQuery as TVirtualDataSet) do begin
            VState  := vsInternalInsert;
            try
              Insert();
              FieldByName(sZLEZLECENIEMAT_ID).AsInteger     := UniqueKey[sZLEZLECENIEMAT];
              FieldByName(sZLEZLECENIE_ID).AsInteger        := iZlecId;
              FieldByName(sLWPRECEPPOZOPER_ID).AsInteger    := iOperId;
              FieldByName(sLWPRECEPPOZOPERZAM_ID).AsInteger := oQuery.FieldByName(sLWPRECEPPOZOPERZAM_ID).AsInteger;
              FieldByName(sASOASORT_ID).AsInteger           := oQuery.FieldByName(sASOASORT_ID).AsInteger;
              FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat    := dIlosc;
              if(not oQuery.FieldByName(sASOASORTKMAT_ID).IsNull) then
                FieldByName(sASOASORTKMAT_ID).AsInteger       := oQuery.FieldByName(sASOASORTKMAT_ID).AsInteger;
              SetTechnicalFieldsOnInsert();
              UpdateModifiedRecordFlagsForAddedRecord();
              for i := 1 to oQuery.Fields.Count-1 do begin
                if((oQuery.Fields[i].FullName = sUNIJM_SYMBOL) and (FindField(sUNIJM_SYMBOL) <> nil)) then begin
                  FieldByName(sUNIJM_SYMBOL).AsString := sJMSymbol
                end else if((oQuery.Fields[i].FullName = sUNIJM_NAZWA) and (FindField(sUNIJM_NAZWA) <> nil)) then begin
                  FieldByName(sUNIJM_NAZWA).AsString := sJMNazwa
                end else if((oQuery.Fields[i].FullName = sUNIJM_ID) and (FindField(sUNIJM_ID) <> nil)) then begin
                  FieldByName(sUNIJM_ID).AsInteger := oQuery.Fields[19].AsInteger
                end else if(FindField(oQuery.Fields[i].FullName) <> nil) then
                  FieldByName(oQuery.Fields[i].FullName).AsString := oQuery.Fields[i].AsString;
              end;  // for i := 1 to oQuery.Fields.Count-1
              Post();
            finally
              VState := vsNormal;
            end;
          end;  // with(DataModule.ActiveQuery as TVirtualDataSet)
          oQuery.Next();
        end;  // while(not oQuery.Eof)
      end;  // if(oQuery <> nil)
    finally
      FreeAndNil(oQuery);
      FreeAndNil(oQueryJM);
      DataModule.ActiveQuery.EnableControls();
    end;
  end;  // procedure WybierzZOperacji

begin
  if(not DMAQFbN(sLWPRECEPPOZMATERIAL_ID).IsNull) then
    WybierzZReceptury()
  else if(not DMAQFbN(sLWPRECEPPOZOPER_ID).IsNull) then
    WybierzZOperacji();
end;

procedure TZLEZlecenieMaterialDBGridForm.PrzywrocMaterialZReceptury();

  procedure PrzywrocZReceptury();
  var
    oQuery:   TUNIQuery;
    oDetail:  TZLEZlecenieMaterialDBDetailForm;
    dIlosc:   double;
  begin
    oQuery  := SQLManager.CreateQuery(
      'select ' +
{0}     'A.'  + sASOASORT_ID        + ', RPM.'+ sLWPRECEPPOZMATERIAL_ILOSC+ ', ' +
{2}     'R.'  + sLWPRECEPTURA_ILOSC + ', A.'  + sASOASORT_SYMBOL          + ', ' +
{4}     'A.'  + sASOASORT_NAZWA     + ', A.'  + sASOASORT_PKWIU           + ', ' +
{6}     'A.'  + sASOASORT_INDEKS    + ', G.'  + sASOGRAS_SYMBOL           + ', ' +
{8}     'G.'  + sASOGRAS_NAZWA      + ', JM.' + sUNIJM_SYMBOL             + ', ' +
{10}    'JM.' + sUNIJM_NAZWA        + ', RP.' + sLWPRECEPPOZ_ID           + ', ' +
{12}    'AKM.'+ sASOASORTKMAT_ID    + ', KM.' + sASOKMAT_ID               + ', ' +
{14}    'KM.' + sASOKMAT_SYMBOL     + ', KM.' + sASOKMAT_NAZWA            + ', ' +
{16}    'RPM.'+ sUNIJM_ID           + ', JMM.'+ sUNIJM_SYMBOL             + ', ' +
{18}    'A.'  + sUNIJM_ID2          + ', A.'  + sASOASORT_WSPJM1JM2       + ', ' +
{20}    'A.'  + sUNIJM_ID3          + ', A.'  + sASOASORT_WSPJM1JM3       + ', ' +
{22}    'A.'  + sUNIJM_ID1          + ', ' +
{23}    'A.'  + sASOASORT_ZJM1NAJM2 + ', A.'  + sASOASORT_ZJM1NAJM3 +
      ' from ' +
        '%s RPM'  +
          ' inner join %s RP' +
            ' on RPM.'+ sLWPRECEPPOZ_ID + ' = RP.'+ sLWPRECEPPOZ_ID +
          ' inner join %s R'  +
            ' on RP.' + sLWPRECEPTURA_ID+ ' = R.' + sLWPRECEPTURA_ID+
          ' inner join %s A'  +
            ' on RPM.'  + sASOASORT_ID  + ' = A.' + sASOASORT_ID    +
          ' left outer join %s G' +
            ' on A.'  + sASOGRAS_ID     + ' = G.' + sASOGRAS_ID     +
          ' left outer join %s JM'  +
            ' on A.'  + sUNIJM_ID1      + ' = JM.'+ sUNIJM_ID       +
          ' left outer join %s AKM' +
            ' on RPM.'+ sASOASORTKMAT_ID+ ' = AKM.'+sASOASORTKMAT_ID+
          ' left outer join %s KM'  +
            ' on AKM.'+ sASOKMAT_ID     + ' = KM.'+ sASOKMAT_ID     +
          ' left outer join %s JMM'  +
            ' on RPM.'+ sUNIJM_ID + ' = JMM.'+ sUNIJM_ID       +
      ' where ' +
        'RPM.'  + sLWPRECEPPOZMATERIAL_ID + ' = ' + DMAQFbN(sLWPRECEPPOZMATERIAL_ID).AsString,
      [
        sLWPRECEPPOZMATERIAL, sLWPRECEPPOZ, sLWPRECEPTURA ,
        sASOASORT           , sASOGRAS    , sUNIJM        ,
        sASOASORTKMAT       , sASOKMAT    , sUNIJM
      ]
    );
    try
      oQuery.Open();
      if(not oQuery.IsEmpty()) then begin
        dIlosc  := (
          ((Owner as TZLEZlecenieDBDetailForm).DMAQFbN(sZLEZLECENIE_ILOSC).AsFloat * oQuery.Fields[1].AsFloat) /
            oQuery.Fields[2].AsFloat
        );
        if(oQuery.Fields[16].AsInteger = oQuery.Fields[22].AsInteger) then begin
            // podstawowa jednostka
          dIlosc  := dIlosc;
        end else if(oQuery.Fields[16].AsInteger = oQuery.Fields[18].AsInteger) then begin
            // druga jednostka
          if(oQuery.Fields[19].AsInteger = 0) then
            raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[3].AsString, '2']));
          if(oQuery.Fields[23].AsString = sASOCONST_KIER_PRZEL1_ID) then
            dIlosc  := dIlosc * oQuery.Fields[19].AsFloat
          else
            dIlosc  := dIlosc / oQuery.Fields[19].AsFloat;
        end else if(oQuery.Fields[16].AsInteger = oQuery.Fields[20].AsInteger) then begin
            // trzecia jednostka
          if(oQuery.Fields[21].AsInteger = 0) then
            raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[3].AsString, '3']));
          if(oQuery.Fields[24].AsString = sASOCONST_KIER_PRZEL1_ID) then
            dIlosc  := dIlosc * oQuery.Fields[21].AsFloat
          else
            dIlosc  := dIlosc / oQuery.Fields[21].AsFloat;
        end else begin
          raise EOF_ZLEException.Create(Format(csError_AsortBrakJednostki, [oQuery.Fields[3].AsString, oQuery.Fields[17].AsString]));
        end;
        oDetail := (FormManager.OpenDBDetail(
          self,
          'TZLEZlecenieMaterialDBDetailForm',
          omEdit + omHidden + omOverVirtual
        ) as TZLEZlecenieMaterialDBDetailForm);
        try
          oDetail.DMAQFbN(sASOASORT_ID).AsInteger       := oQuery.Fields[0].AsInteger;
          oDetail.DMAQFbN(sZLEZLECENIEMAT_ILOSC).AsFloat:= dIlosc;
          if(not oQuery.Fields[12].IsNull) then
            oDetail.DMAQFbN(sASOASORTKMAT_ID).AsInteger := oQuery.Fields[12].AsInteger
          else
            oDetail.DMAQFbN(sASOASORTKMAT_ID).Clear();
          oDetail.ShowAsort(oQuery);
          oDetail.ShowKontoMaterialowe(oQuery);
          oDetail.ShowModal();
        finally
          oDetail.Free();
        end;
      end;  // if(not oQuery.IsEmpty())
    finally
      FreeAndNil(oQuery);
    end;
  end;  // procedure PrzywrocZReceptury

  procedure PrzywrocZOperacji();
  var
    iOperId:    integer;
    iZlecId:    integer;
    i:          integer;
    oQuery:     TUNIQuery;
    sZlecIlosc: string;
    dIlosc:     double;
  begin
      // najpierw usuwamy aktualne rekordy z biecej operacji
    oQuery  := nil;
    DataModule.ActiveQuery.DisableControls();
    try
      iOperId := DMAQFbN(sLWPRECEPPOZOPER_ID).AsInteger;
      iZlecId := DMAQFbN(sZLEZLECENIE_ID).AsInteger;
      DataModule.ActiveQuery.First();
      while(not DataModule.ActiveQuery.Eof) do begin
        if(DMAQFbN(sLWPRECEPPOZOPER_ID).AsInteger = iOperId) then begin
          DataModule.ActiveQuery.Delete();
        end else
          DataModule.ActiveQuery.Next();
      end;  // while(not DataModule.ActiveQuery.Eof)
        // dopisujemy materiay operacji
      sZlecIlosc  := (Owner as TZLEZlecenieDBDetailForm).DMAQFbN(sZLEZLECENIE_ILOSC).AsString;
      oQuery  := SQLManager.CreateQuery(
        'select ' +
  {0}     'A.'  + sASOASORT_ID        + ', '  +
  {1}     '((' + sZlecIlosc + ' * OPM.'+ sLWPRECEPPOZOPERMAT_ILOSC + ') / R.' + sLWPRECEPTURA_ILOSC + ') as ' + sZLEZLECENIEMAT_ILOSC + ', ' +
  {2}     'A.'  + sASOASORT_SYMBOL          + ', ' +
  {3}     'A.'  + sASOASORT_NAZWA     + ', A.'  + sASOASORT_PKWIU           + ', ' +
  {5}     'A.'  + sASOASORT_INDEKS    + ', G.'  + sASOGRAS_SYMBOL           + ', ' +
  {7}     'G.'  + sASOGRAS_NAZWA      + ', JM.' + sUNIJM_SYMBOL             + ', ' +
  {9}     'JM.' + sUNIJM_NAZWA        + ', RP.' + sLWPRECEPPOZ_ID           + ', ' +
  {11}    'AKM.'+ sASOASORTKMAT_ID    + ', KM.' + sASOKMAT_ID               + ', ' +
  {13}    'KM.' + sASOKMAT_SYMBOL     + ', KM.' + sASOKMAT_NAZWA            + ', ' +
  {15}    'OP.' + sLWPRECEPPOZOPER_NAZWA  + ', OP.' + sLWPRECEPPOZOPER_OPIS + ', ' +
  {17}    sZlecIlosc + ' as ' + sZLEZLECENIE_ILOSC  + ', ' +
  {18}    'OPM.'+ sUNIJM_ID           + ', JMM.'+ sUNIJM_SYMBOL             + ', ' +
  {20}    'A.'  + sUNIJM_ID2          + ', A.'  + sASOASORT_WSPJM1JM2       + ', ' +
  {22}    'A.'  + sUNIJM_ID3          + ', A.'  + sASOASORT_WSPJM1JM3       + ', ' +
  {24}    'A.'  + sUNIJM_ID1          + ', ' +
  {25}    'A.'  + sASOASORT_ZJM1NAJM2 + ', A.'  + sASOASORT_ZJM1NAJM3 +
        ' from ' +
          '%s OP'  +
            ' inner join %s RP' +
              ' on OP.'+ sLWPRECEPPOZ_ID + ' = RP.'+ sLWPRECEPPOZ_ID +
            ' inner join %s R'  +
              ' on RP.' + sLWPRECEPTURA_ID+ ' = R.' + sLWPRECEPTURA_ID+
            ' inner join %s OPM' +
              ' on OP.'+ sLWPRECEPPOZOPER_ID + ' = OPM.'+ sLWPRECEPPOZOPER_ID +
            ' inner join %s A'  +
              ' on OPM.'  + sASOASORT_ID  + ' = A.' + sASOASORT_ID    +
            ' left outer join %s G' +
              ' on A.'  + sASOGRAS_ID     + ' = G.' + sASOGRAS_ID     +
            ' left outer join %s JM'  +
              ' on A.'  + sUNIJM_ID1      + ' = JM.'+ sUNIJM_ID       +
            ' left outer join %s AKM' +
              ' on OPM.'+ sASOASORTKMAT_ID+ ' = AKM.'+sASOASORTKMAT_ID+
            ' left outer join %s KM'  +
              ' on AKM.'+ sASOKMAT_ID     + ' = KM.'+ sASOKMAT_ID     +
            ' left outer join %s JMM'  +
              ' on OPM.'  + sUNIJM_ID + ' = JMM.' + sUNIJM_ID +
        ' where ' +
          'OP.'  + sLWPRECEPPOZOPER_ID + ' = ' + IntToStr(iOperId),
        [
          sLWPRECEPPOZOPER  , sLWPRECEPPOZ, sLWPRECEPTURA ,
          sLWPRECEPPOZOPERMAT,
          sASOASORT         , sASOGRAS, sUNIJM        ,
          sASOASORTKMAT     , sASOKMAT, sUNIJM
        ]
      );
      oQuery.Open();
      while(not oQuery.Eof) do begin
        dIlosc  := oQuery.FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat;
        if(oQuery.Fields[18].AsInteger = oQuery.Fields[24].AsInteger) then begin
            // podstawowa jednostka
          dIlosc  := dIlosc;
        end else if(oQuery.Fields[18].AsInteger = oQuery.Fields[20].AsInteger) then begin
            // druga jednostka
          if(oQuery.Fields[21].AsInteger = 0) then
            raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[2].AsString, '2']));
          if(oQuery.Fields[25].AsString = sASOCONST_KIER_PRZEL1_ID) then
            dIlosc  := dIlosc * oQuery.Fields[21].AsFloat
          else
            dIlosc  := dIlosc / oQuery.Fields[21].AsFloat;
        end else if(oQuery.Fields[18].AsInteger = oQuery.Fields[22].AsInteger) then begin
            // trzecia jednostka
          if(oQuery.Fields[23].AsInteger = 0) then
            raise EOF_ZLEException.Create(Format(csError_AsortBrakWspolczynnika, [oQuery.Fields[2].AsString, '3']));
          if(oQuery.Fields[26].AsString = sASOCONST_KIER_PRZEL1_ID) then
            dIlosc  := dIlosc * oQuery.Fields[23].AsFloat
          else
            dIlosc  := dIlosc / oQuery.Fields[23].AsFloat;
        end else begin
          raise EOF_ZLEException.Create(Format(csError_AsortBrakJednostki, [oQuery.Fields[2].AsString, oQuery.Fields[19].AsString]));
        end;
        with(DataModule.ActiveQuery as TVirtualDataSet) do begin
          VState  := vsInternalInsert;
          try
            Insert();
            FieldByName(sZLEZLECENIEMAT_ID).AsInteger     := UniqueKey[sZLEZLECENIEMAT];
            FieldByName(sZLEZLECENIE_ID).AsInteger        := iZlecId;
            FieldByName(sLWPRECEPPOZOPER_ID).AsInteger    := iOperId;
            FieldByName(sASOASORT_ID).AsInteger           := oQuery.FieldByName(sASOASORT_ID).AsInteger;
            FieldByName(sZLEZLECENIEMAT_ILOSC).AsFloat    := dIlosc;
            if(not oQuery.FieldByName(sASOASORTKMAT_ID).IsNull) then
              FieldByName(sASOASORTKMAT_ID).AsInteger       := oQuery.FieldByName(sASOASORTKMAT_ID).AsInteger;
            SetTechnicalFieldsOnInsert();
            UpdateModifiedRecordFlagsForAddedRecord();
            for i := 1 to oQuery.Fields.Count-1 do begin
              if((oQuery.Fields[i].FullName <> sZLEZLECENIEMAT_ILOSC) and (FindField(oQuery.Fields[i].FullName) <> nil)) then
                FieldByName(oQuery.Fields[i].FullName).AsString := oQuery.Fields[i].AsString;
            end;  // for i := 1 to oQuery.Fields.Count-1
            Post();
          finally
            VState := vsNormal;
          end;
        end;  // with(DataModule.ActiveQuery as TVirtualDataSet)
        oQuery.Next();
      end;  // while(not oQuery.Eof)
    finally
      FreeAndNil(oQuery);
      DataModule.ActiveQuery.EnableControls();
    end;
  end;  // PrzywrocZOperacji

begin
  if(Komunikat(cConfirmation, 'Przywrci materiay z receptury?', MB_YESNO + MB_ICONQUESTION) = mrYes) then begin
    if(not DMAQFbN(sLWPRECEPPOZMATERIAL_ID).IsNull) then
      PrzywrocZReceptury()
    else if(not DMAQFbN(sLWPRECEPPOZOPER_ID).IsNull) then
      PrzywrocZOperacji();
  end;
end;

function  TZLEZlecenieMaterialDBGridForm.MoznaWybracZamiennik(): boolean;
begin
  result  := (
    (not DMAQFbN(sLWPRECEPPOZMATERIAL_ID).IsNull) or
    (not DMAQFbN(sLWPRECEPPOZOPER_ID).IsNull)
  );
end;

function  TZLEZlecenieMaterialDBGridForm.MoznaPrzywrocicMaterial():  boolean;
begin
    // dla aktualnego rekordu materia mozna przywrci z receptury jeeli
    // jest to materia operacji lub
    // jest wypenione pole sLWPRECEPPOZMATERIAL_ID (czyli mona dosta si do receptury)
    // i asortyment zosta zmieniony (aktualny rekord ma inny asortyment ni odpowiadajcy mu materia pozycji receptury)
  result  := (
    (not DMAQFbN(sLWPRECEPPOZOPER_ID).IsNull) or
    (
      (not DMAQFbN(sLWPRECEPPOZMATERIAL_ID).IsNull) and
      (DMAQFbN(sASOASORT_ID).AsInteger <> DMAQFbN(sLWPAlias_ASOAsort).AsInteger)
    )
  );
end;

{
  Wywietla detaila materiau i zwraca czy uytkownik zatwierdzi go czy te anulowa.
  Zwraca TRUE jeeli uytkownik zatwierdzi formatk przez Zatwierd.
}
function  TZLEZlecenieMaterialDBGridForm.EdytujMaterial(): boolean;
var
  oDetail:  TZLEZlecenieMaterialDBDetailForm;
begin
  oDetail := (FormManager.OpenDBDetail(
    self,
    'TZLEZlecenieMaterialDBDetailForm',
    omEdit + omHidden + omOverVirtual
  ) as TZLEZlecenieMaterialDBDetailForm);
  try
    result  := (oDetail.ShowModal() = mrOk);
  finally
    oDetail.Free();
  end;
end;

function  TZLEZlecenieMaterialDBGridForm.CreateDataModule(
  const AOpenMode:  TOpenMode
): TUNIDataModule;
begin
  result  := TZLEZlecenieMaterialDataModule.CreateOpen(
      self,
      DefaultMasterKeyValues(),
      AOpenMode
  );
end;

procedure TZLEZlecenieMaterialDBGridForm.View;
begin
  FormManager.OpenDBDetail(
    self,
    'TZLEZlecenieMaterialDBDetailForm',
    omView + omShowModal + omOverVirtual
  );
end;

procedure TZLEZlecenieMaterialDBGridForm.New;
begin
  FormManager.OpenDBDetail(
    self,
    'TZLEZlecenieMaterialDBDetailForm',
    omNew + omShowModal + omOverVirtual
  );
end;

procedure TZLEZlecenieMaterialDBGridForm.Open;
begin
  FormManager.OpenDBDetail(
    self,
    'TZLEZlecenieMaterialDBDetailForm',
    omEdit + omShowModal + omOverVirtual
  );
end;

procedure TZLEZlecenieMaterialDBGridForm.Delete;
begin
  FormManager.OpenDBDetail(
    self,
    'TZLEZlecenieMaterialDBDetailForm',
    omDelete + omShowModal + omOverVirtual
  );
end;

procedure TZLEZlecenieMaterialDBGridForm.KopiujMatZRecepturyMenuItemClick(
  Sender: TObject);
var
  oParams:      TKeyValues;
  sZlecenieId:  string;
  sRecepturaId: string;
  sZlecIlosc:   string;
begin
  sZlecenieId := '';
  if((Parameters = nil) or (not ParamIsNotEmpty(Parameters, sZLEZLECENIE_ID, sZlecenieId))) then begin
    oParams := TKeyValues.CreateEmpty();
    try
      if(Owner is TUNIForm) then begin
        (Owner as TUNIForm).GetParametersForForm(self, oParams);
        ParamIsNotEmpty(oParams, sZLEZLECENIE_ID, sZlecenieId);
        ParamIsNotEmpty(oParams, sLWPRECEPTURA_ID, sRecepturaId);
        ParamIsNotEmpty(oParams, sZLEZLECENIE_ILOSC, sZlecIlosc);
      end;  // if(Owner is TUNIForm)
    finally
      oParams.Free();
    end;
  end;  // if((Parameters = nil) or (not ParamIsNotEmpty(Parameters, sZLEZLECENIE_ID, sZlecenieId)))
  Assert(Length(sZlecenieId) > 0, 'Nie podano zlecenia. [KopiujMatZReceptury]');
  Assert(Length(sRecepturaId) > 0, 'Nie podano receptury. [KopiujMatZReceptury]');
  Assert(Length(sZlecIlosc) > 0, 'Nie podano iloci zlecenia. [KopiujMatZReceptury]');
  KopiujMaterialyZReceptury(
    StrToInt(sZlecenieId),
    StrToInt(sRecepturaId),
    StrToFloat(sZlecIlosc),
    false
  );
end;

procedure TZLEZlecenieMaterialDBGridForm.ZamiennikMenuItemClick(
  Sender: TObject);
begin
  WybierzZamiennik();
end;

procedure TZLEZlecenieMaterialDBGridForm.OrygMaterialMenuItemClick(
  Sender: TObject);
begin
  PrzywrocMaterialZReceptury();
end;

procedure TZLEZlecenieMaterialDBGridForm.DataSourceDataChange(
  Sender: TObject; Field: TField);
begin
  inherited;
  ZamiennikMenuItem.Enabled   := MoznaWybracZamiennik();
  OrygMaterialMenuItem.Enabled:= MoznaPrzywrocicMaterial();
end;

procedure TZLEZlecenieMaterialDBGridForm.DBGridDrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
begin
  if(
    (Column.Field <> nil)
    and (
      (Column.Field.FullName = sZLEZLECENIEMAT_ILOSC)
      or (Column.Field.FullName = sZAOZAPOTRZPOZ_ILOSC)
      or (Column.Field.FullName = sZAOZAPOTPOZZAMWYP_ILOSC)
      or (Column.Field.FullName = csILOSC_PZ)
      or (Column.Field.FullName = csILOSC_RW)
      or (Column.Field.FullName = 'STANDYSP')
      or (Column.Field.FullName = 'STANMAG')
    )
  ) then begin
    if(DMAQFbN(sASOASORT_FORMAT_ILOSC).IsNull) then
      DBGrid.DefaultDrawColumnCell(Rect, DataCol, Column, State)
    else
      DBGrid.WriteText(DBGrid.Canvas, Rect, FormatFloat(UNIGetMaskFromFormat(DMAQFbN(sASOASORT_FORMAT_ILOSC).AsFloat, false), Column.Field.AsFloat), taRightJustify);
  end else
    DBGrid.DefaultDrawColumnCell(Rect, DataCol, Column, State);
end;

initialization
  FormManager.RegisterForm(TZLEZlecenieMaterialDBGridForm, sZLEModulID);
end.
