unit ZLEZlecenieMaterialUnit;
{
 Copyright Unisoft Sp. z o.o.

Changes
---------------------------------------------------------------------------------------------
When       Who      What
---------------------------------------------------------------------------------------------
2007-09-11 Adam     Utworzone
2007-11-14 Adam     Dodane stany dysponowane i magazynowe materiaw
2008-08-26 Adam     Dodane pola operacji i aktualnego zamiennika operacji
2012-03-23 Adam     Moliwo dopisywania materiaw do zlece nieprodukcyjnych
2012-06-06 Adam     Dodane kolumny zapotrzebowano, zamwiono, dostarczono, pobrano
---------------------------------------------------------------------------------------------}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIDataModuleUnit, Db, UNIComponentsUnit, DBTables;

type
  TZLEZlecenieMaterialDataModule = class(TUNIDataModule)
  protected
    procedure GetKeys();  override;
    function  GetTableName(): string; override;
    procedure InitializeGridQueryDefs();  override;

  private
    m_sZlecenieId:      string;
    m_iZlecenieIndeks:  integer;

    procedure ComposeConditions();

  end;

const
  csILOSC_PZ  = 'ILOSC_PZ';
  csILOSC_RW  = 'ILOSC_RW';

implementation
uses
  QueryBuilderUnit,
  UNIStaleUnit,
  SQLManagerUnit,
  UNIDataManagerUnit,
  Dodatki,
  KeyManagerUnit,
  UNIFormUnit,
  ZAODefUnit,
  MAGDefUnit,
  UNISysDefUnit,
  ASODefUnit,
  LWPDefUnit,
  ZLEDefUnit
  ;

{$R *.DFM}

procedure TZLEZlecenieMaterialDataModule.GetKeys;
begin
  FKeys.AddPrimary(sZLEZLECENIEMAT_ID);
end;

function  TZLEZlecenieMaterialDataModule.GetTableName: string;
begin
  Result  := sZLEZLECENIEMAT;
end;

procedure TZLEZlecenieMaterialDataModule.InitializeGridQueryDefs;
var
  sMAGSQL:  string;
begin
  with GridColDefs do begin
    Clear();
    Add(sASOASORT_SYMBOL  , 'A' , 'Symbol', true, true, taLeftJustify, 100);
    Add(sASOASORT_NAZWA   , 'A' , 'Nazwa', true, true, taLeftJustify, 300);
    Add(sASOASORT_INDEKS  , 'A' , 'Indeks', true, true, taLeftJustify, 100);
    Add(sASOASORT_PKWIU   , 'A' , 'PKWIU', true, true, taLeftJustify, 100);
    Add(sZLEZLECENIEMAT_ILOSC, sZLEALIAS_ZlecenieMaterial, 'Ilo', True, True, taRightJustify, 100, uftCounter);
    Add(sASOGRAS_SYMBOL   , 'G' , 'Symbol grupy', True, True, taLeftJustify, 70);
    Add(sASOGRAS_NAZWA    , 'G' , 'Nazwa grupy', True, True, taLeftJustify, 70);
    Add(sUNIJM_SYMBOL     , 'J' , 'JM', True, True, taLeftJustify, 50);
    Add(
      SQLManager.GetTableName(sMAG_SP_MAGZWROCSTANDYSP) + '(A.' + sASOASORT_ID + ');STANDYSP',
      '',
      'Stan dysponowany', true, true, taRightJustify, 100, uftCounter
    );
    Add(
      SQLManager.GetTableName(sMAG_SP_MAGZWROCSTANIL) + '(A.' + sASOASORT_ID + ');STANMAG',
      '',
      'Stan magazynowy', true, true, taRightJustify, 100, uftCounter
    );
    Add(sASOKMAT_SYMBOL , 'KM' , 'Symbol konta materiaowego', True, True, taLeftJustify, 100);
    Add(sASOKMAT_NAZWA  , 'KM' , 'Nazwa konta materiaowego', false, True, taLeftJustify, 200);

    if(DataManagers.Available(sLWPModulID)) then begin
      Add(sLWPRECEPPOZOPER_NAZWA, sLWPAlias_RecepturaPozOper, 'Nazwa operacji', true, true, taLeftJustify, 100);
      Add(sLWPRECEPPOZOPER_OPIS, sLWPAlias_RecepturaPozOper, 'Opis operacji', false, true, taLeftJustify, 200, uftString);
      Add(sLWPRECEPPOZOPERZAM_NAZWA, sLWPAlias_RecepturaPozOperZam, 'Nazwa wybranego zamiennika operacji', true, true, taLeftJustify, 100);
      Add(sLWPRECEPPOZOPERZAM_OPIS, sLWPAlias_RecepturaPozOperZam, 'Opis wybranego zamiennika operacji', false, true, taLeftJustify, 200, uftString);
      AddRequiredAndUnvisible(sLWPRECEPPOZOPER_NAZWA, sLWPAlias_RecepturaPozOper, '');
      AddRequiredAndUnvisible(sLWPRECEPPOZOPERZAM_NAZWA, sLWPAlias_RecepturaPozOperZam, '');
      AddRequiredAndUnvisible(sLWPRECEPPOZMATERIAL_ID, sLWPAlias_RecepturaPozMaterial, '');
      AddRequiredAndUnvisible(
        'nvl(' +
          sLWPAlias_RecepturaPoz + '.' + sLWPRECEPPOZ_ID + ', ' +
          sLWPAlias_RecepturaPoz + '2.' + sLWPRECEPPOZ_ID +
        ');' + sLWPRECEPPOZ_ID,
        '', ''
      );
      AddRequiredAndUnvisible(
        'nvl(' +
          sLWPAlias_RecepturaPoz + '.' + sLWPRECEPTURA_ID + ', ' +
          sLWPAlias_RecepturaPoz + '2.' + sLWPRECEPTURA_ID +
        ');' + sLWPRECEPTURA_ID,
        '', ''
      );
      AddRequiredAndUnvisible(sASOASORT_ID + ';' + sLWPAlias_ASOAsort, 'A2', '' );
    end;
    if(DataManagers.Available(sZAOModulID)) then begin
      Add(sZAOZAPOTRZPOZ_ILOSC, sZLEALIAS_ZlecenieMaterial, 'Zapotrzebowano', false, true, taLeftJustify, 100, uftCounter);
      Add(sZAOZAPOTPOZZAMWYP_ILOSC, sZLEALIAS_ZlecenieMaterial, 'Zamwiono', false, true, taLeftJustify, 100, uftCounter);
    end;
    Add(csILOSC_PZ, sZLEALIAS_ZlecenieMaterial, 'Dostarczono', false, true, taLeftJustify, 100, uftCounter);
    Add(csILOSC_RW, sZLEALIAS_ZlecenieMaterial, 'Pobrano', false, true, taLeftJustify, 100, uftCounter);

    AddRequiredAndUnvisible(sASOASORT_SYMBOL  , 'A' , '');
    AddRequiredAndUnvisible(sASOASORT_NAZWA   , 'A' , '');
    AddRequiredAndUnvisible(sASOASORT_INDEKS  , 'A' , '');
    AddRequiredAndUnvisible(sASOASORT_PKWIU   , 'A' , '');
    AddRequiredAndUnvisible(sZLEZLECENIEMAT_ILOSC, sZLEALIAS_ZlecenieMaterial, '');
    AddRequiredAndUnvisible(sASOGRAS_SYMBOL   , 'G' , '');
    AddRequiredAndUnvisible(sASOGRAS_NAZWA    , 'G' , '');
    AddRequiredAndUnvisible(sUNIJM_SYMBOL     , 'J' , '');
    AddRequiredAndUnvisible(sZLEZLECENIE_ILOSC, sZLEALIAS_Zlecenie, '');

    AddRequiredAndUnvisible(sASOKMAT_SYMBOL , 'KM' , '');
    AddRequiredAndUnvisible(sASOKMAT_NAZWA  , 'KM' , '');
    AddRequiredAndUnvisible(sASOKMAT_ID     , 'KM' , '');

    AddRequiredAndUnvisible(sASOASORT_ID, 'A', '' );
    AddRequiredAndUnvisible(sASOASORT_FORMAT_ILOSC, 'A', '');

    AddRequiredAndUnvisible(sZLEZLECENIEMAT_ID, sZLEALIAS_ZlecenieMaterial, '');
  end;
  with GridTabDefs do begin
    Clear();
    sMAGSQL :=
      ' LEFT OUTER JOIN (' +
        'SELECT ' +
          'OP.' + sZLEZLECENIE_ID + ', OP.' + sASOASORT_ID + ', ' +
          'Sum(OP.' + sMAGOBRPOZ_ILOSC + ' * OP.' + sMAGOBRPOZ_ZNAK + ') AS ' + csILOSC_PZ +
        ' FROM ' +
          SQLManager.GetTableName(sMAGModulID + cMAGOBRPOZ_SUFFIX) + ' OP' +
            ' INNER JOIN ' + SQLManager.GetTableName(sMAGModulID + cMAGOBR_SUFFIX) + ' O' +
              ' ON OP.' + sMAGOBR_ID + ' = O.' + sMAGOBR_ID +
            ' INNER JOIN ' + SQLManager.GetTableName(sMAGTYPDOK) + ' T' +
              ' ON O.' + sMAGTYPDOK_ID + ' = T.' + sMAGTYPDOK_ID +
        ' WHERE T.' + sMAGCONST_PROTOTYP_ID + ' = ''PZ''' +
        ' GROUP BY ' +
          'OP.' + sZLEZLECENIE_ID + ', OP.' + sASOASORT_ID +
      ') PZ' +
        ' ON (' +
          'ZM.' + sZLEZLECENIE_ID + ' = PZ.' + sZLEZLECENIE_ID +
          ' AND ZM.' + sASOASORT_ID + ' = PZ.' + sASOASORT_ID +
        ')' +
      ' LEFT OUTER JOIN (' +
        'SELECT ' +
          'OP.' + sZLEZLECENIE_ID + ', OP.' + sASOASORT_ID + ', ' +
          'Sum(' +
            'CASE ' +
              'WHEN T.' + sMAGCONST_PROTOTYP_ID + ' = ''ZU'' THEN (OP.' + sMAGOBRPOZ_ILOSC + ' * OP.' + sMAGOBRPOZ_ZNAK + ') ' +
              'WHEN T.' + sMAGCONST_PROTOTYP_ID + ' = ''ZZ'' THEN -(OP.' + sMAGOBRPOZ_ILOSC + ' * OP.' + sMAGOBRPOZ_ZNAK + ')' +
            'END' +
          ') AS ' + csILOSC_RW +
        ' FROM ' +
          SQLManager.GetTableName(sMAGModulID + cMAGOBRPOZ_SUFFIX) + ' OP' +
            ' INNER JOIN ' + SQLManager.GetTableName(sMAGModulID + cMAGOBR_SUFFIX) + ' O' +
              ' ON OP.' + sMAGOBR_ID + ' = O.' + sMAGOBR_ID +
            ' INNER JOIN ' + SQLManager.GetTableName(sMAGTYPDOK) + ' T' +
              ' ON O.' + sMAGTYPDOK_ID + ' = T.' + sMAGTYPDOK_ID +
        ' WHERE T.' + sMAGCONST_PROTOTYP_ID + ' in(''ZU'', ''ZZ'')' +
        ' GROUP BY ' +
          'OP.' + sZLEZLECENIE_ID + ', OP.' + sASOASORT_ID +
      ') RW' +
        ' ON (' +
          'ZM.' + sZLEZLECENIE_ID + ' = RW.' + sZLEZLECENIE_ID +
          ' AND ZM.' + sASOASORT_ID + ' = RW.' + sASOASORT_ID +
        ')'
    ;
    if(DataManagers.Available(sZAOModulID)) then begin
      Add(
        sZLEALIAS_ZlecenieMaterial,
        '(' +
          'SELECT ' +
            'ZP.' + sZAOZAPOTRZPOZ_ILOSC + ', ZP.' + sZAOZAPOTPOZZAMWYP_ILOSC + ', ' +
            'PZ.' + csILOSC_PZ + ', RW.' + csILOSC_RW + ', ' +
            'ZM.*' +
          ' FROM ' +
            SQLManager.GetTableName(GetTableName()) + ' ZM' +
              ' LEFT OUTER JOIN (' +
                'SELECT ' +
                  'T.' + sZLEZLECENIE_ID          + ', T.' + sASOASORT_ID + ', ' +
                  'Sum(T.' + sZAOZAPOTPOZZAMWYP_ILOSC + ') AS ' + sZAOZAPOTPOZZAMWYP_ILOSC + ', ' +
                  'Sum(T.' + sZAOZAPOTRZPOZ_ILOSC + ') AS ' + sZAOZAPOTRZPOZ_ILOSC +
                ' FROM ' +
                  '(' +
                    'SELECT ' +
                      'ZZ.' + sZLEZLECENIE_ID + ', ZP2.' + sASOASORT_ID + ', ' +
                      'Nvl(ZAMP.' + sZAOZAPOTPOZZAMWYP_ILOSC + ', 0) AS ' + sZAOZAPOTPOZZAMWYP_ILOSC + ', ' +
                      'ZP2.' + sZAOZAPOTRZPOZ_ILOSC +
                    ' FROM ' +
                      '(' +
                        'SELECT DISTINCT ' +
                          'ZZ2.' + sZAOZAPOTRZPOZ_ID + ', ZZ2.' + sZLEZLECENIE_ID +
                        ' FROM ' + SQLManager.GetTableName(sZAOZAPOTRZPOZZLEC) + ' ZZ2' +
                        ' WHERE ' +
                          'ZZ2.' + sZAOZAPOTRZPOZZLEC_KIERUNEK + ' = ' + IntToStr(cZAPOTRZ_KIERUNEK_ZLECZAPOTRZ) +
                      ') ZZ' +
                        ' INNER JOIN ' + SQLManager.GetTableName(sZAOZAPOTRZPOZ) + ' ZP2' +
                          ' ON ZZ.' + sZAOZAPOTRZPOZ_ID + ' = ZP2.' + sZAOZAPOTRZPOZ_ID +
                        ' LEFT OUTER JOIN (' +
                          'SELECT ' +
                            'ZAPPZAMP.' + sZAOZAPOTRZPOZ_ID + ', ZAMP2.' + sASOASORT_ID + ', ' +
                            'Sum(ZAPPZAMP.' + sZAOZAPOTPOZZAMWYP_ILOSC + ') AS ' + sZAOZAPOTPOZZAMWYP_ILOSC +
                          ' FROM ' +
                            '(' +
                              'SELECT DISTINCT ' +
                                'ZAPPZAMP2.' + sZAOZAPOTPOZZAMWYP_ILOSC + ', ' +
                                'ZAPPZAMP2.' + sZAOZAPOTRZPOZ_ID + ', ZAPPZAMP2.' + sZAOZAMOWIENIEWYPOZ_ID +
                              ' FROM ' + SQLManager.GetTableName(sZAOZAPOTPOZZAMWYP) + ' ZAPPZAMP2' +
                            ') ZAPPZAMP' +
                              ' INNER JOIN ' + SQLManager.GetTableName(sZAOZAMOWIENIEWYPOZ) + ' ZAMP2' +
                                ' ON ZAPPZAMP.' + sZAOZAMOWIENIEWYPOZ_ID + ' = ZAMP2.' + sZAOZAMOWIENIEWYPOZ_ID +
                          ' GROUP BY ' +
                            'ZAPPZAMP.' + sZAOZAPOTRZPOZ_ID + ', ZAMP2.' + sASOASORT_ID +
                        ') ZAMP' +
                          ' ON (' +
                              'ZP2.' + sZAOZAPOTRZPOZ_ID + ' = ZAMP.' + sZAOZAPOTRZPOZ_ID +
                              ' AND ZP2.' + sASOASORT_ID + ' = ZAMP.' + sASOASORT_ID +
                          ')' +
                  ') T' +
                ' GROUP BY ' +
                  'T.' + sZLEZLECENIE_ID + ', T.' + sASOASORT_ID +
              ') ZP' +
                ' ON (' +
                  'ZM.' + sZLEZLECENIE_ID + ' = ZP.' + sZLEZLECENIE_ID +
                  ' AND ZM.' + sASOASORT_ID + ' = ZP.' + sASOASORT_ID +
                ')' +
              sMAGSQL +
        ')',
        '',
        false
      );
    end else begin
      Add(
        sZLEALIAS_ZlecenieMaterial,
        '(' +
          'SELECT ' +
            'PZ.' + csILOSC_PZ + ', RW.' + csILOSC_RW + ', ' +
            'ZM.*' +
          ' FROM ' +
            SQLManager.GetTableName(GetTableName()) + ' ZM' +
              sMAGSQL +
        ')',
        '',
        false
      );
    end;
    Add(sZLEALIAS_Zlecenie            , sZLEZLECENIE);
    if(DataManagers.Available(sLWPModulID)) then begin
      Add(sLWPAlias_RecepturaPozMaterial, sLWPRECEPPOZMATERIAL);
      Add(sLWPAlias_RecepturaPozOper    , sLWPRECEPPOZOPER);
      Add(sLWPAlias_RecepturaPozOperZam , sLWPRECEPPOZOPERZAM);
      Add(sLWPAlias_RecepturaPoz        , sLWPRECEPPOZ);
      Add(sLWPAlias_RecepturaPoz+'2'    , sLWPRECEPPOZ);
      Add('A2'                          , sASOASORT);
    end;
    Add('A'                           , sASOASORT);
    Add('G'                           , sASOGRAS);
    Add('J'                           , sUNIJM);
    Add('KM'                          , sASOKMAT);
    Add('AKM'                         , sASOASORTKMAT);
  end;
  with GridJoinDefs do begin
    Clear();
    AddExpres(
      sZLEALIAS_ZlecenieMaterial + '.' + sZLEZLECENIE_ID + ' = ' +
        sZLEALIAS_ZLECENIE + '.' + sZLEZLECENIE_ID,
      qjkInner,
      false
    );
    if(DataManagers.Available(sLWPModulID)) then begin
      AddExpres(
        sZLEALIAS_ZlecenieMaterial + '.' + sLWPRECEPPOZMATERIAL_ID + ' = ' +
          sLWPAlias_RecepturaPozMaterial + '.' + sLWPRECEPPOZMATERIAL_ID,
        qjkLeftOuter,
        false
      );
      AddExpres(
        sLWPAlias_RecepturaPozMaterial + '.' + sLWPRECEPPOZ_ID + ' = ' +
          sLWPAlias_RecepturaPoz + '.' + sLWPRECEPPOZ_ID,
        qjkLeftOuter,
        true  // true aby framework zaadowa tabel sLWPAlias_RecepturaPoz
      );
      AddExpres(
        sZLEALIAS_ZlecenieMaterial + '.' + sLWPRECEPPOZOPER_ID + ' = ' +
          sLWPAlias_RecepturaPozOper + '.' + sLWPRECEPPOZOPER_ID,
        qjkLeftOuter,
        true  // true aby framework zaadowa tabel sLWPAlias_RecepturaPoz2
      );
      AddExpres(
        sZLEALIAS_ZlecenieMaterial + '.' + sLWPRECEPPOZOPERZAM_ID + ' = ' +
          sLWPAlias_RecepturaPozOperZam + '.' + sLWPRECEPPOZOPERZAM_ID,
        qjkLeftOuter,
        false
      );
      AddExpres(
        sLWPAlias_RecepturaPozOper + '.' + sLWPRECEPPOZ_ID + ' = ' +
          sLWPAlias_RecepturaPoz + '2.' + sLWPRECEPPOZ_ID,
        qjkLeftOuter,
        true  // true aby framework zaadowa tabel sLWPAlias_RecepturaPoz2
      );
      AddExpres(sLWPAlias_RecepturaPozMaterial + '.' + sASOASORT_ID + ' = A2.' + sASOASORT_ID, qjkLeftOuter, false);
    end;
    AddExpres(sZLEALIAS_ZlecenieMaterial + '.' + sASOASORT_ID + ' = A.' + sASOASORT_ID, qjkInner, false);
    AddExpres('A.'  + sASOGRAS_ID + ' = G.' + sASOGRAS_ID, qjkLeftOuter, false);
    AddExpres('A.'  + sUNIJM_ID1  + ' = J.' + sUNIJM_ID, qjkLeftOuter, false);

    AddExpres(
      sZLEALIAS_ZlecenieMaterial + '.' + sASOASORTKMAT_ID + ' = AKM.' + sASOASORTKMAT_ID,
      qjkLeftOuter,
      false
    );
    AddExpres(
      'AKM.'  + sASOKMAT_ID + ' = KM.' + sASOKMAT_ID,
      qjkLeftOuter,
      false
    );

    m_iZlecenieIndeks := AddEmptyCondition();
  end;
  ComposeConditions();

  inherited;
end;

procedure TZLEZlecenieMaterialDataModule.ComposeConditions();
var
  oParams:        TKeyValues;
begin
  if(not ParamIsNotEmpty(MasterKeyValues, sZLEZLECENIE_ID, m_sZlecenieId)) then begin
    oParams := TKeyValues.CreateEmpty();
    try
      (Owner as TUNIForm).GetParametersForForm((Owner as TUNIForm), oParams);
      ParamIsNotEmpty(oParams, sZLEZLECENIE_ID, m_sZlecenieId);
    finally
      oParams.Free();
    end;
  end;  // if nie ma id zlecenia w MasterKeyValues
  if(Length(m_sZlecenieId) > 0) then begin
    GridJoinDefs.ExchangeCondition(
      m_iZlecenieIndeks,
      sZLEALIAS_ZlecenieMaterial + '.' + sZLEZLECENIE_ID  + ' = ' + m_sZlecenieId,
      true
    );
  end else begin
    GridJoinDefs.ExchangeCondition(
      m_iZlecenieIndeks,
      '',
      false
    );
  end;
end;

end.
