unit ZLEZlecenieWycenaDBGridFormUnit;
{
 Copyright Unisoft Sp. z o.o.

Changes
---------------------------------------------------------------------------------------------
When       Who      What
---------------------------------------------------------------------------------------------
2008-01-11 Adam     Utworzone
2008-02-28 Adam     Utworzona funkcja ZLEDM.AktualizujWyceneZlecen
2008-08-28 Adam     Dodana funkcja akceptacji i cofania akceptacji
---------------------------------------------------------------------------------------------}
interface

uses
  UNIStaleUnit,  UNIDataManagerUnit,  UNIDataModuleUnit, UNIFormUnit, KeyManagerUnit,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIDBGridFormUnit, Db, DBTables, UNIComponentsUnit, URDataSet,
  UniDesigner, Menus, Grids, DBGrids, StdCtrls, ComCtrls, DBCtrls, ToolWin,
  ExtCtrls;

type
  TZLEZlecenieWycenaDBGridForm = class(TUNIDBGridForm)
    FunctionsPanel: TUNIPanel;
    PokazButton: TUNIButton;
    ZmienButton: TUNIButton;
    AktualizujButton: TUNIButton;
    AkceptacjaButton: TUNIButton;
    AkceptacjaCofnijButton: TUNIButton;
    procedure AktualizujButtonClick(Sender: TObject);
    procedure PokazButtonClick(Sender: TObject);
    procedure ZmienButtonClick(Sender: TObject);
    procedure AkceptacjaButtonClick(Sender: TObject);
    procedure AkceptacjaCofnijButtonClick(Sender: TObject);
    procedure DataSourceDataChange(Sender: TObject; Field: TField);
  public
    procedure GetParametersForForm(
                  a_oForm:           TUNIForm;
                  a_oParameters:     TKeyValues
              ); override;

  protected
    function  CreateDataModule(
                  const AOpenMode:  TOpenMode
              ): TUNIDataModule; override;
    procedure SetUserRights();  override;
    procedure View(); override;
    procedure Open(); override;

  private
    procedure AktualizujRekordy(
                    const ac_iZlecenieId: integer
              );

  end;

implementation
uses
  FormManagerUnit,
  SecurityManagerUnit,
  SQLManagerUnit,
  UniqueKeyUnit,
  Dodatki,
  ZLEZlecenieWycenaUnit,
  ZLEDataManagerUnit,
  LWPDefUnit,
  ZLEDefUnit
  ;

{$R *.DFM}

procedure TZLEZlecenieWycenaDBGridForm.GetParametersForForm(
  a_oForm:           TUNIForm;
  a_oParameters:     TKeyValues
);
begin
  if(Owner is TUNIDBGridForm) then
    (Owner as TUNIDBGridForm).GetParametersForForm(self, a_oParameters);
end;

function  TZLEZlecenieWycenaDBGridForm.CreateDataModule(
  const AOpenMode:  TOpenMode
): TUNIDataModule;
begin
  result  := TZLEZlecenieWycenaDataModule.CreateOpen(
      self,
      nil,
      AOpenMode
  );
  Switch_New_HotKey(false);
  Switch_Delete_HotKey(false);
end;

procedure TZLEZlecenieWycenaDBGridForm.SetUserRights();
begin
  with(SecurityManager) do begin
    ZmienButton.Enabled := SM_UserCanDo(sZLE_KART_ZLECEN_WYCENA_ZMIEN);
    PokazButton.Enabled := SM_UserCanDo(sZLE_KART_ZLECEN_WYCENA_POKAZ);
    Switch_Open_HotKey(PokazButton.Enabled);
    Switch_View_HotKey(PokazButton.Enabled);
    AktualizujButton.Enabled:= SM_UserCanDo(sZLE_KART_ZLECEN_WYCENA_AKTUALIZUJ);
    AkceptacjaButton.Enabled:= SM_UserCanDo(sZLE_KART_ZLECEN_WYCENA_AKCEPTUJ);
    AkceptacjaCofnijButton.Enabled:= SM_UserCanDo(sZLE_KART_ZLECEN_WYCENA_COFNIJ_AKCEPT);
  end;  // with(SecurityManager)
end;

procedure TZLEZlecenieWycenaDBGridForm.View();
begin
  FormManager.OpenDBDetail(
      self,
      'TZLEZlecenieWycenaDBDetailForm',
      omView + omShowModal
  );
end;

procedure TZLEZlecenieWycenaDBGridForm.Open();
begin
  FormManager.OpenDBDetail(
      self,
      'TZLEZlecenieWycenaDBDetailForm',
      omEdit + omShowModal
  );
end;

procedure TZLEZlecenieWycenaDBGridForm.AktualizujRekordy(
  const ac_iZlecenieId: integer
);
begin
  ZLEDM.AktualizujWyceneZlecen(IntToStr(ac_iZlecenieId));
  DataModule.RefreshDataSets();
end;

procedure TZLEZlecenieWycenaDBGridForm.AktualizujButtonClick(
  Sender: TObject);
var
  sZlecenieId:  string;
  oParams:      TKeyValues;
begin
  if(not (Komunikat(cConfirmation, 'Zaktualizowa dane o pracy zasobw?', MB_YESNO + MB_ICONQUESTION) = mrYes)) then
    exit;
  sZlecenieId := '';
  if(not DataModule.ActiveQuery.IsEmpty()) then
    sZlecenieId := DMAQFbN(sZLEZLECENIE_ID).AsString
  else begin
    oParams := TKeyValues.CreateEmpty();
    try
      sZlecenieId := oParams.Values[sZLEZLECENIE_ID];
      (Owner as TUNIForm).GetParametersForForm(self, oParams);
      sZlecenieId := oParams.Values[sZLEZLECENIE_ID];
    finally
      oParams.Free();
    end;
  end;
  if(Length(sZlecenieId) > 0) then
    AktualizujRekordy(StrToInt(sZlecenieId));
end;

procedure TZLEZlecenieWycenaDBGridForm.PokazButtonClick(Sender: TObject);
begin
  ViewMenuItemClick(self);
  DBGrid.SetFocus();
end;

procedure TZLEZlecenieWycenaDBGridForm.ZmienButtonClick(Sender: TObject);
begin
  EditMenuItemClick(self);
  DBGrid.SetFocus();
end;

procedure TZLEZlecenieWycenaDBGridForm.AkceptacjaButtonClick(
  Sender: TObject);
var
  iResult:  integer;
begin
  if(not DataModule.ActiveQuery.IsEmpty()) then begin
    iResult := Komunikat(cConfirmation, 'Zmieni status zaznaczonych rekordw na ' + cZLEConstZleWycenaStatus_Zaakceptowany + '?', MB_YESNO + MB_ICONQUESTION);
    if(iResult = mrYes) then begin
      if(Length(DBGrid.MarkedRows) > 0) then
        ZLEDM.AkceptujWyceneZlecen(DBGrid.MarkedRows)
      else
        ZLEDM.AkceptujWyceneZlecen([DMAQFbN(sZLEZLECENIEWYC_ID).AsInteger]);
      DataModule.RefreshDataSets();
    end;  // if(iResult = mrYes)
  end;  // if(not DataModule.ActiveQuery.IsEmpty())
end;

procedure TZLEZlecenieWycenaDBGridForm.AkceptacjaCofnijButtonClick(
  Sender: TObject);
var
  iResult:  integer;
begin
  if(not DataModule.ActiveQuery.IsEmpty()) then begin
    iResult := Komunikat(cConfirmation, 'Zmieni status zaznaczonych rekordw na ' + cZLEConstZleWycenaStatus_Wprowadzony + '?', MB_YESNO + MB_ICONQUESTION);
    if(iResult = mrYes) then begin
      if(Length(DBGrid.MarkedRows) > 0) then
        ZLEDM.CofnijAkceptacjeWycenyZlecen(DBGrid.MarkedRows)
      else
        ZLEDM.CofnijAkceptacjeWycenyZlecen([DMAQFbN(sZLEZLECENIEWYC_ID).AsInteger]);
      DataModule.RefreshDataSets();
    end;  // if(iResult = mrYes)
  end;  // if(not DataModule.ActiveQuery.IsEmpty())
end;

procedure TZLEZlecenieWycenaDBGridForm.DataSourceDataChange(
  Sender: TObject; Field: TField);
begin
  inherited;
  ZmienButton.Enabled := (
    SecurityManager.SM_UserCanDo(sZLE_KART_ZLECEN_WYCENA_ZMIEN) and
    (DMAQFbN(sZLEZLECENIEWYC_STATUS).AsString <> cZLEConstZleWycenaStatus_ZaakceptowanyID)
  );
  Switch_Open_HotKey(ZmienButton.Enabled);
end;

initialization
  FormManager.RegisterForm(TZLEZlecenieWycenaDBGridForm, sZLEModulID);
end.
