unit ZLEZlecenieZasobDBGridFormUnit;
{
 Copyright Unisoft Sp. z o.o.

Changes
---------------------------------------------------------------------------------------------
When       Who      What
---------------------------------------------------------------------------------------------
2005-02-23 gdeyk    Utworzone
2007-11-14 Adam     Dodane przeliczanie obcienia zasobu zalenie od wielkoci zlecenia
2008-08-26 Adam     Poprawione kopiowanie zasobw z receptury
}
interface

uses
  UNIFormUnit, KeyManagerUnit,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UNIDBGridFormUnit, Db, DBTables, UNIComponentsUnit, URDataSet,
  UniDesigner, Menus, Grids, DBGrids, StdCtrls, ComCtrls, DBCtrls, ToolWin,
  ExtCtrls, UNIStaleUnit, UNIDataModuleUnit;

type
  TZLEZlecenieZasobDBGridForm = class(TUNIDBGridForm)
    EditButtonsPanel: TUNIPanel;
    DopiszButton: TUNIButton;
    UsunButton: TUNIButton;
    ProdukcjaPanel: TUNIPanel;
    ZasKopiujButton: TUNIButton;
    ZasZamiennikButton: TUNIButton;
    ZasOryginalButton: TUNIButton;
    procedure DopiszButtonClick(Sender: TObject);
    procedure UsunButtonClick(Sender: TObject);
    procedure ZasKopiujButtonClick(Sender: TObject);
    procedure ZasZamiennikButtonClick(Sender: TObject);
    procedure ZasOryginalButtonClick(Sender: TObject);
  private
    procedure InitializeControls();
    procedure DodajZasoby();
    procedure UsunZasoby();
    procedure _AfterScroll(DataSet: TDataSet);
    procedure SkopiujZasobyReceptury(
                  const ac_iReceptura_Id:   integer;
                  const ac_iZlecenieId:     integer;
                  const ac_dZlecenieIlosc:  double
              );

  protected
    procedure New(); override;
    procedure Delete(); override;
    function CreateDataModule(const AOpenMode: TOpenMode): TUNIDataModule; override;
    procedure BeforeShow(); override;
  public
    procedure InternalOpen(); override;
    procedure PrzepiszZasobyZReceptury(
                  const ac_bProsOPotwierdzenie: boolean
              );
      {
        Pozwala wybra zamiennik dla aktualnego zasobu.
        Lista zamiennikw brana jest z receptury
      }
    procedure WybierzZamiennik();
      {
        Przywraca dla aktualnej pozycji zasb z receptury.
      }
    procedure PrzywrocZasobZReceptury();

  end;

implementation

{$R *.DFM}

uses
  Dodatki,
  FormManagerUnit,
  UNIDataManagerUnit,
  UNIDBDetailFormUnit,
  UniqueKeyUnit,
  SQLManagerUnit,
  LWPDataManagerExtUnit,
  ZLEDefUnit,
  ZLEDataManagerUnit,
  ZLEZlecenieZasobUnit,
  LWPDefUnit;

{ TZLEZlecenieZasobDBGridForm }

function TZLEZlecenieZasobDBGridForm.CreateDataModule(
  const AOpenMode: TOpenMode): TUNIDataModule;
begin
  result := TZLEZlecenieZasobDataModule.CreateOpen( Self, InheritedMasterKeyValues, AOpenMode );
end;

procedure TZLEZlecenieZasobDBGridForm.BeforeShow();
begin
  inherited;
  ProdukcjaPanel.Visible  := (
    DataManagers.Available(sLWPModulID) and
    (Owner <> nil) and
    (LowerCase(Owner.ClassName) = 'tzlezleceniedbdetailform')
  );
end;

procedure TZLEZlecenieZasobDBGridForm.Delete;
begin
  UsunZasoby();
end;

procedure TZLEZlecenieZasobDBGridForm.DodajZasoby;
var
  oQuery: TUNIQuery;
begin
  oQuery := nil;
  try
    oQuery := ZLEDM.ZLEPokazIObsluzDane( sZLEZASOB,
                                         sZLEALIAS_ZLEZASOB_ID+';'+
                                         sZLEALIAS_ZLEZASOB_NAZWA+';'+
                                         sZLEALIAS_ZLEZASOB_STAWKA+';'+
                                         sZLEALIAS_ZLEZASOB_RODZAJ+';'+
                                         sZLEALIAS_RODZAJZASOBU_V,
                                         '', False, True );
    if oQuery <> nil then begin
      while not oQuery.Eof do begin
        with (DataModule.ActiveQuery as TVirtualDataSet) do begin
          if not Locate( sZLEZASOB_ID, oQuery.FieldByName(sZLEZASOB_ID).AsInteger, []) then
            try
              VState := vsInternalInsert;
              Append;
              DMAQFbN(sZLEZLECENIEZASOB_ID).AsInteger := UniqueKey[sZLEZLECENIEZASOB];
              DMAQFbN(sZLEZLECENIE_ID).AsString := Parameters.Values[sZLEZLECENIE_ID];
              DMAQFbN(sZLEZASOB_ID).AsInteger := oQuery.FieldByName(sZLEZASOB_ID).AsInteger;
              SetTechnicalFieldsOnInsert();
              UpdateModifiedRecordFlagsForAddedRecord();
              DMAQFbN(sZLEZASOB_NAZWA).AsString := oQuery.FieldByName(sZLEZASOB_NAZWA).AsString;
              DMAQFbN(sZLEZASOB_STAWKA).AsFloat := oQuery.FieldByName(sZLEZASOB_STAWKA).AsFloat;
              DMAQFbN(sZLEZASOB_RODZAJ).AsString := oQuery.FieldByName(sZLEZASOB_RODZAJ).AsString;
              DMAQFbN(sZLECONST_RODZAJZASOBU_V).AsString := oQuery.FieldByName(sZLECONST_RODZAJZASOBU_V).AsString;
              Post;
            finally
              VState := vsNormal;
            end;
        end;
        oQuery.Next;
      end;
    end;
  finally
    if oQuery <> nil then oQuery.Free;
  end;

end;

procedure TZLEZlecenieZasobDBGridForm.InitializeControls;
begin
  Switch_Open_HotKey(False);
  Switch_View_HotKey(False);
  Switch_Print_Option(False);
  EditButtonsPanel.Visible := (OpenModeFlags and omReadOnly) = 0;
end;

procedure TZLEZlecenieZasobDBGridForm.InternalOpen;
begin
  if Parameters = nil then
    Parameters := TKeyValues.CreateEmpty();
  InitializeControls();
  inherited;
  DataModule.ActiveQuery.AfterScroll := _AfterScroll;
  _AfterScroll(DataModule.ActiveQuery);
end;

procedure TZLEZlecenieZasobDBGridForm.PrzepiszZasobyZReceptury(
  const ac_bProsOPotwierdzenie: boolean
);
var
  bUsun:  boolean;
begin
  if(LowerCase(Owner.ClassName) = 'tzlezleceniedbdetailform') then begin
    if(not (Owner as TUNIDBDetailForm).DMAQFbN(sLWPRECEPTURA_ID).IsNull) then begin
      if(
        (not ac_bProsOPotwierdzenie) or
        (Komunikat(cConfirmation, 'Przywrci materiay z receptury?', MB_YESNO + MB_ICONQUESTION) = mrYes)
      ) then begin
        bUsun := false;
        if(not DataModule.ActiveQuery.IsEmpty()) then begin
          bUsun := (Komunikat(
            cConfirmation,
            'Usuna istniejce zasoby ze zlecenia?',
            MB_YESNO + MB_DEFBUTTON1 + MB_ICONQUESTION
          ) = mrYes);
        end;
        if(bUsun) then begin
          DataModule.ActiveQuery.First();
          while(not DataModule.ActiveQuery.Eof) do begin
            DataModule.ActiveQuery.Delete();
          end;
        end;  // if(bUsun)
        SkopiujZasobyReceptury(
          (Owner as TUNIDBDetailForm).DMAQFbN(sLWPRECEPTURA_ID).AsInteger,
          (Owner as TUNIDBDetailForm).DMAQFbN(sZLEZLECENIE_ID).AsInteger,
          (Owner as TUNIDBDetailForm).DMAQFbN(sZLEZLECENIE_ILOSC).AsFloat
        );
      end;  // if potwierdzono kopiowanie
    end;  // if(not (Owner as TUNIDBDetailForm).DMAQFbN(sLWPRECEPTURA_ID).IsNull)
  end;  // if(LowerCase(Owner.ClassName) = 'tzlezleceniedbdetailform')
end;

procedure TZLEZlecenieZasobDBGridForm.WybierzZamiennik();
var
  oQuery:   TUNIQuery;
  i:        integer;
  oParams:  TKeyValues;
begin
  oParams := nil;
  if((Owner <> nil) and (LowerCase(Owner.ClassName) = 'tzlezleceniedbdetailform')) then begin
    oParams := TKeyValues.CreateAs(
      [
        sLWPRECEPTURA_ID + '=' + (Owner as TUNIDBDetailForm).DMAQFbN(sLWPRECEPTURA_ID).AsString
      ]
    );
  end;
  if(not DMAQFbN(sLWPRECEPPOZZASOB_ID).IsNull) then begin
    oQuery  := LWPDM.LWPPokazIObsluzDane(sLWPRECEPPOZZASOBZAM,
{0}   sLWPALIAS_ZLEZASOBZAM_ID    + ';' + sLWPALIAS_ZLEZASOBZAM_NAZWA + ';' +
{2}   sLWPALIAS_ZLEZASOBZAM_STAWKA+ ';' + sLWPALIAS_ZLEZASOBZAM_RODZAJ+ ';' +
{4}   sLWPAlias_RecepturaPozZasob_Id  + ';' + sLWPAlias_RecepturaPozZasobZam_Id + ';' +
{6}   sLWPAlias_RecepturaPozZasobZam_Plan + ';' + sLWPAlias_RecepturaPozZasobZam_Obciaz + ';' +
{8}   sLWPAlias_ZasobZamJednCzas_Id   + ';' + sLWPAlias_RecepturaPozZasobZam_Wyd+ ';' +
{10}  sLWPAlias_RecepturaPozZasobZam_WydJed + ';' + sLWPAlias_ZasobZamJednCzas_Symbol + ';' +
{12}  sLWPAlias_ZasobZamJednCzas_Nazwa+ ';' + sLWPAlias_ZasobZamWydJednCzas_Symbol    + ';' +
{14}  sLWPAlias_ZasobZamWydJednCzas_Nazwa,
      '@' + sLWPAlias_RecepturaPozZasob_Id  + ' = ' + DMAQFbN(sLWPRECEPPOZZASOB_ID).AsString,
      true, true, false, '', nil, oParams
    );
    if(oQuery <> nil) then begin
      with(DataModule.ActiveQuery as TVirtualDataSet) do begin
        VState  := vsInternalInsert;
        try
          Edit();
          FieldByName(sZLEZASOB_ID).AsInteger := oQuery.Fields[0].AsInteger;
          FieldByName(sZLEZLECENIEZASOB_PLAN).AsString  := oQuery.Fields[6].AsString;
          if(not oQuery.Fields[7].IsNull) then begin
            FieldByName(sZLEZLECENIEZASOB_OBCIAZ).AsFloat := oQuery.Fields[7].AsFloat;
            if(not oQuery.Fields[8].IsNull) then
              FieldByName(sLWPJEDNCZAS_ID).AsInteger  := oQuery.Fields[8].AsInteger
            else
              FieldByName(sLWPJEDNCZAS_ID).Clear();
          end else begin
            FieldByName(sZLEZLECENIEZASOB_OBCIAZ).Clear();
            FieldByName(sLWPJEDNCZAS_ID).Clear();
          end;
          if(not oQuery.Fields[9].IsNull) then begin
            FieldByName(sZLEZLECENIEZASOB_WYD).AsFloat := oQuery.Fields[9].AsFloat;
            if(not oQuery.Fields[10].IsNull) then
              FieldByName(sZLEZLECENIEZASOB_WYDJED).AsInteger := oQuery.Fields[10].AsInteger
            else
              FieldByName(sZLEZLECENIEZASOB_WYDJED).Clear();
          end else begin
            FieldByName(sZLEZLECENIEZASOB_WYD).Clear();
            FieldByName(sZLEZLECENIEZASOB_WYDJED).Clear();
          end;
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZASOB_ID));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_PLAN));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_OBCIAZ));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sLWPJEDNCZAS_ID));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_WYD));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_WYDJED));
          if(ModifiedRecordsFlagsList.GetRecordFlag() = mrfNotModified) then
            ModifiedRecordsFlagsList.SetRecordFlag(mrfModified);
            // ustawiamy dane jednostki wydajnoci
          if(FindField(sLWPAlias_ZasobWydJednCzas_Symbol) <> nil) then
              FieldByName(sLWPAlias_ZasobWydJednCzas_Symbol).AsString := oQuery.Fields[13].AsString;
          if(FindField(sLWPAlias_ZasobWydJednCzas_Nazwa) <> nil) then
              FieldByName(sLWPAlias_ZasobWydJednCzas_Nazwa).AsString := oQuery.Fields[14].AsString;
            // ustawiamy dane jednostki obcienia
          if(FindField(sLWPJEDNCZAS_SYMBOL) <> nil) then
              FieldByName(sLWPJEDNCZAS_SYMBOL).AsString := oQuery.Fields[11].AsString;
          if(FindField(sLWPJEDNCZAS_NAZWA) <> nil) then
              FieldByName(sLWPJEDNCZAS_NAZWA).AsString := oQuery.Fields[12].AsString;
            // odwieamy pierwsze 9 pl (liczc od 0) bo reszta jest odwieona wczeniej
          for i := 1 to 8 do begin
            if(FindField(oQuery.Fields[i].FullName) <> nil) then
              FieldByName(oQuery.Fields[i].FullName).AsString := oQuery.Fields[i].AsString;
          end;  // for i := 1 to 8
          Post();
        finally
          VState := vsNormal;
        end;
      end;  // with(DataModule.ActiveQuery as TVirtualDataSet)
    end;  // if(oQuery <> nil)
  end;  // if(not DMAQFbN(sLWPRECEPPOZMATERIAL_ID).IsNull)
end;

procedure TZLEZlecenieZasobDBGridForm.PrzywrocZasobZReceptury();
var
  oQuery:   TUNIQuery;
  i:        integer;
begin
  if(not DMAQFbN(sLWPRECEPPOZZASOB_ID).IsNull) then begin
    oQuery  := LWPDM.LWPOdpytajODane(sLWPRECEPPOZZASOB,
{0}   sLWPALIAS_ZLEZASOB_ID     + ';' + sLWPALIAS_ZLEZASOB_NAZWA  + ';' +
{2}   sLWPALIAS_ZLEZASOB_STAWKA + ';' + sLWPALIAS_ZLEZASOB_RODZAJ + ';' +
{4}   sLWPAlias_RecepturaPozZasob_Id  + ';' + sLWPAlias_RecepturaPozZasob_Plan  + ';' +
{6}   sLWPAlias_RecepturaPozZasob_Obciaz+ ';' + sLWPAlias_ZasobJednCzas_Id + ';' +
{8}   sLWPAlias_RecepturaPozZasob_Wyd   + ';' + sLWPAlias_RecepturaPozZasob_WydJed+ ';' +
{10}  sLWPAlias_ZasobJednCzas_Symbol    + ';' + sLWPAlias_ZasobJednCzas_Nazwa     + ';' +
{12}  sLWPAlias_ZasobWydJednCzas_Symbol + ';' + sLWPAlias_ZasobWydJednCzas_Nazwa,
      '@' + sLWPAlias_RecepturaPozZasob_Id  + ' = ' + DMAQFbN(sLWPRECEPPOZZASOB_ID).AsString,
      false
    );
    if(oQuery <> nil) then begin
      with(DataModule.ActiveQuery as TVirtualDataSet) do begin
        VState  := vsInternalInsert;
        try
          Edit();
          FieldByName(sZLEZASOB_ID).AsInteger := oQuery.Fields[0].AsInteger;
          FieldByName(sZLEZLECENIEZASOB_PLAN).AsString  := oQuery.Fields[5].AsString;
          if(not oQuery.Fields[6].IsNull) then begin
            FieldByName(sZLEZLECENIEZASOB_OBCIAZ).AsFloat := oQuery.Fields[6].AsFloat;
            if(not oQuery.Fields[7].IsNull) then
              FieldByName(sLWPJEDNCZAS_ID).AsInteger  := oQuery.Fields[7].AsInteger
            else
              FieldByName(sLWPJEDNCZAS_ID).Clear();
          end else begin
            FieldByName(sZLEZLECENIEZASOB_OBCIAZ).Clear();
            FieldByName(sLWPJEDNCZAS_ID).Clear();
          end;
          if(not oQuery.Fields[8].IsNull) then begin
            FieldByName(sZLEZLECENIEZASOB_WYD).AsFloat := oQuery.Fields[8].AsFloat;
            if(not oQuery.Fields[9].IsNull) then
              FieldByName(sZLEZLECENIEZASOB_WYDJED).AsInteger := oQuery.Fields[9].AsInteger
            else
              FieldByName(sZLEZLECENIEZASOB_WYDJED).Clear();
          end else begin
            FieldByName(sZLEZLECENIEZASOB_WYD).Clear();
            FieldByName(sZLEZLECENIEZASOB_WYDJED).Clear();
          end;
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZASOB_ID));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_PLAN));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_OBCIAZ));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sLWPJEDNCZAS_ID));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_WYD));
          ModifiedRecordsFlagsList.SetFieldIsModified(FieldByName(sZLEZLECENIEZASOB_WYDJED));
          ModifiedRecordsFlagsList.SetRecordFlag(mrfModified);

            // ustawiamy dane jednostki wydajnoci
          if(FindField(sLWPAlias_ZasobWydJednCzas_Symbol) <> nil) then
              FieldByName(sLWPAlias_ZasobWydJednCzas_Symbol).AsString := oQuery.Fields[12].AsString;
          if(FindField(sLWPAlias_ZasobWydJednCzas_Nazwa) <> nil) then
              FieldByName(sLWPAlias_ZasobWydJednCzas_Nazwa).AsString := oQuery.Fields[13].AsString;
            // ustawiamy dane jednostki obcienia
          if(FindField(sLWPJEDNCZAS_SYMBOL) <> nil) then
              FieldByName(sLWPJEDNCZAS_SYMBOL).AsString := oQuery.Fields[10].AsString;
          if(FindField(sLWPJEDNCZAS_NAZWA) <> nil) then
              FieldByName(sLWPJEDNCZAS_NAZWA).AsString := oQuery.Fields[11].AsString;
            // odwieamy pierwsze 8 pl (liczc od 0) bo reszta jest odwieona wczeniej
          for i := 1 to 7 do begin
            if(FindField(oQuery.Fields[i].FullName) <> nil) then
              FieldByName(oQuery.Fields[i].FullName).AsString := oQuery.Fields[i].AsString;
          end;  // for i := 1 to 7
          Post();
        finally
          VState := vsNormal;
        end;
      end;  // with(DataModule.ActiveQuery as TVirtualDataSet)
    end;  // if(oQuery <> nil)
  end;  // if(not DMAQFbN(sLWPRECEPPOZMATERIAL_ID).IsNull)
end;

procedure TZLEZlecenieZasobDBGridForm.New;
begin
  DodajZasoby();
end;

procedure TZLEZlecenieZasobDBGridForm.UsunZasoby;
begin
  if Komunikat( cConfirmation,'Usunicie zasobu spowoduje usunicie wszystkich'#13'pozycji rozlicze powizanych z tym zasobem.'#13'Czy jeste pewny, e chcesz usun ten zasb?',MB_YESNO+MB_DEFBUTTON2)=idYes then begin
     try

        SQLManager.ExecuteQuery('DELETE FROM %s WHERE ' + sZLEZASOBYPRACA_ID + ' IN (SELECT zr.' +
                          sZLEZASOBYPRACA_ID + ' FROM %s  z, %s zp, %s zr ' +
                          ' WHERE z.' + sZLEZLECENIE_ID + '=zp.' + sZLEZLECENIE_ID + ' AND zp.' +
                          sZLEZLECENIE_ID + '=' + DMAQFbN(sZLEZLECENIE_ID).AsString + ' AND zr.' +
                          sZLEZLECENIEPOZ_ID + '=zp.' + sZLEZLECENIEPOZ_ID + ' AND zr.' +
                          sZLEZASOB_ID + '=' + DMAQFbN(sZLEZASOB_ID).AsString + ')',
                          [sZLEZASOBYPRACA, sZLEZLECENIE, sZLEZLECENIEPOZ, sZLEZASOBYPRACA]);
     except
        Komunikat(cError, 'Blad kasowania pozycji rozliczen');
        raise;
     end;

     DataModule.ActiveQuery.Delete();
  end;

end;

procedure TZLEZlecenieZasobDBGridForm._AfterScroll(DataSet: TDataSet);
begin
  DopiszButton.Enabled := NewMenuItem.Enabled;
  UsunButton.Enabled := DeleteMenuItem.Enabled;
end;

procedure TZLEZlecenieZasobDBGridForm.SkopiujZasobyReceptury(
  const ac_iReceptura_Id:   integer;
  const ac_iZlecenieId:     integer;
  const ac_dZlecenieIlosc:  double
);

  procedure DodajZasob(
    const ac_oQuery:  TUNIQuery
  );
  var
    i:  integer;
  begin
    with(DataModule.ActiveQuery as TVirtualDataSet) do begin
      VState  := vsInternalInsert;
      try
        Insert();
        FieldByName(sZLEZLECENIEZASOB_ID).AsInteger   := UniqueKey[sZLEZLECENIEZASOB];
        FieldByName(sZLEZLECENIE_ID).AsInteger        := ac_iZlecenieId;
        FieldByName(sZLEZASOB_ID).AsInteger           := ac_oQuery.Fields[0].AsInteger;
        FieldByName(sLWPRECEPPOZZASOB_ID).AsInteger   := ac_oQuery.Fields[6].AsInteger;
        FieldByName(sZLEZLECENIEZASOB_PLAN).AsString  := ac_oQuery.Fields[7].AsString;
        if(not ac_oQuery.Fields[8].IsNull) then begin
          FieldByName(sZLEZLECENIEZASOB_OBCIAZ).AsFloat:= ac_oQuery.Fields[8].AsFloat;
          if(not ac_oQuery.Fields[10].IsNull) then
            FieldByName(sLWPJEDNCZAS_ID).AsInteger  := ac_oQuery.Fields[10].AsInteger;
        end;  // if(not ac_oQuery.Fields[8].IsNull)
        if(not ac_oQuery.Fields[11].IsNull) then begin
          FieldByName(sZLEZLECENIEZASOB_WYD).AsFloat:= ac_oQuery.Fields[11].AsFloat;
          if(not ac_oQuery.Fields[12].IsNull) then
            FieldByName(sZLEZLECENIEZASOB_WYDJED).AsInteger := ac_oQuery.Fields[12].AsInteger;
        end;  // if wypeniona wydajno zasobu
        SetTechnicalFieldsOnInsert();
        UpdateModifiedRecordFlagsForAddedRecord();
          // odwieenie jednostki czasu wydajnoci
        if(FindField(sLWPAlias_ZasobWydJednCzas_Symbol) <> nil) then
          FieldByName(sLWPAlias_ZasobWydJednCzas_Symbol).AsString := ac_oQuery.Fields[13].AsString;
        if(FindField(sLWPAlias_ZasobWydJednCzas_Nazwa) <> nil) then
          FieldByName(sLWPAlias_ZasobWydJednCzas_Nazwa).AsString  := ac_oQuery.Fields[14].AsString;
          // odwieenie jednostki czasu obcienia
        if(FindField(sLWPJEDNCZAS_SYMBOL) <> nil) then
          FieldByName(sLWPJEDNCZAS_SYMBOL).AsString := ac_oQuery.Fields[15].AsString;
        if(FindField(sLWPJEDNCZAS_NAZWA) <> nil) then
          FieldByName(sLWPJEDNCZAS_NAZWA).AsString  := ac_oQuery.Fields[16].AsString;
          // odwieenie kolejnoci faz produkcji
        if(FindField(sLWPRECEPPOZ_KOLEJNOSC) <> nil) then
          FieldByName(sLWPRECEPPOZ_KOLEJNOSC).AsInteger := ac_oQuery.Fields[17].AsInteger;
          // odwieamy tylko do 9go pola (pola liczone od 0) bo reszta bya odwieona wyej
        for i := 1 to 8 do begin
          if((FindField(ac_oQuery.Fields[i].FullName) <> nil) and
          (FieldByName(ac_oQuery.Fields[i].FullName).IsNull)) then
            FieldByName(ac_oQuery.Fields[i].FullName).AsString := ac_oQuery.Fields[i].AsString;
        end;  // for i := 1 to ac_oQuery.Fields.Count-1
        Post();
      finally
        VState := vsNormal;
      end;
    end;  // with(DataModule.ActiveQuery as TVirtualDataSet)
  end;  // procedure DodajZasob

var
  oQuery: TUNIQuery;
begin
  oQuery  := LWPDM.LWPOdpytajODane(sLWPRECEPPOZ,
{0} sLWPALIAS_ZLEZASOB_ID     + ';' + sLWPALIAS_ZLEZASOB_NAZWA  + ';' +
{2} sLWPALIAS_ZLEZASOB_STAWKA + ';' + sLWPALIAS_ZLEZASOB_RODZAJ + ';' +
{4} sLWPAlias_RecepturaPoz_Id + ';' + sLWPAlias_Receptura_Id    + ';' +
{6} sLWPAlias_RecepturaPozZasob_Id  + ';' + sLWPAlias_RecepturaPozZasob_Plan+ ';' +
{8} sLWPAlias_RecepturaPozZasob_Obciaz  + ';' + sLWPAlias_Receptura_Ilosc   + ';' +
{10}sLWPAlias_ZasobJednCzas_Id+ ';' + sLWPAlias_RecepturaPozZasob_Wyd       + ';' +
{12}sLWPAlias_ZasobWydJednCzas_Id   + ';' + sLWPAlias_ZasobWydJednCzas_Symbol + ';' +
{14}sLWPAlias_ZasobWydJednCzas_Nazwa+ ';' + sLWPAlias_ZasobJednCzas_Symbol  + ';' +
{16}sLWPAlias_ZasobJednCzas_Nazwa   + ';' + sLWPAlias_RecepturaPoz_Kolejnosc,
    '@' + sLWPAlias_Receptura_Id + ' = ' + IntToStr(ac_iReceptura_Id) + ' and ' +
    '@' + sLWPAlias_RecepturaPozZasob_Id + ' is not null',
    false
  );
  if(oQuery <> nil) then begin
    while(not oQuery.Eof)do begin
      DodajZasob(oQuery);
      oQuery.Next();
    end;  // while(not oQuery.Eof)
    oQuery.Free();
  end;  // if(oQuery <> nil)
end;

procedure TZLEZlecenieZasobDBGridForm.DopiszButtonClick(Sender: TObject);
begin
  New();
end;

procedure TZLEZlecenieZasobDBGridForm.UsunButtonClick(Sender: TObject);
begin
  Delete();
end;

procedure TZLEZlecenieZasobDBGridForm.ZasKopiujButtonClick(
  Sender: TObject);
begin
  PrzepiszZasobyZReceptury(true);
end;

procedure TZLEZlecenieZasobDBGridForm.ZasZamiennikButtonClick(
  Sender: TObject);
begin
  WybierzZamiennik();
end;

procedure TZLEZlecenieZasobDBGridForm.ZasOryginalButtonClick(
  Sender: TObject);
begin
  PrzywrocZasobZReceptury();
end;

initialization
  FormManager.RegisterForm( TZLEZlecenieZasobDBGridForm, sZLEModulID );  

end.
